/* Calculate a DA outcome */
#include <mex.h>
#include <math.h>

/* #define MAXDUB mxGetInf() */
#define MAXDUB INFINITY

/* Function call [match,cutoff_pl,cutoff_fl]=da_alg(u (JxN) ,v (NxJ) ,qty (Jx1) , qty_pl (Jx1), qty_fl(Jx1) , stu_pl_ty(Nx1) , elig(NxJ)) */

/* Requires correct dimensions and full ranks, and \sum qt>=N */
/* J : Number of programs */
/* N : Number of students */
/* u : 'utility' of students - row j column i is u_ij */
/* v : 'utility' of programs - row i column j is p_ij */
/* qty : Jx1 vector of quantities */
/* qty_pl : Jx1 vector of paid lunch quantities */
/* qty_fl : Jx1 vector of free lunch quantities */
/* stu_pl_ty : Nx1 vector of student paid lunch indicator */

void mexFunction(const int nlhs, mxArray *plhs[],const int nrhs, const mxArray *prhs[])
{
  int N,J,K,i,j,app_count, app_pl_count, app_fl_count, appl_id;
  double *u, *v, *u_sort, *v_sort, *qty, *qty_pl, *qty_fl, *app_pointer, *cutoff_pl, *cutoff_fl, cutoff_sch;
  bool any_reject,*rejected, *stu_pl_ty, *elig;

  /* Variable Type Asserts */
  if(!mxIsDouble(prhs[0]) | !mxIsDouble(prhs[1]) | !mxIsDouble(prhs[2]) | !mxIsDouble(prhs[3]) | !mxIsDouble(prhs[4]))
    mexErrMsgIdAndTxt("Dalg:InputType","Error! u, v, qty all need to double!");
  if(!mxIsLogical(prhs[5]) | !mxIsLogical(prhs[6]))
    mexErrMsgIdAndTxt("Dalg:InputType","Error! stu_pl_type and elig need to be logical!");

  /* Dimensions Asserts*/
  J=mxGetM(prhs[0]);
  N=mxGetN(prhs[0]);

  /* u and v sizes */
  K=mxGetM(prhs[1]);
  if(K!=N) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetN(prhs[1]);
  if(K!=J) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  /* qty sizes */
  K=mxGetM(prhs[2]);
  if(K!=J) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetM(prhs[3]);
  if(K!=J) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetM(prhs[4]);
  if(K!=J) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  /* eligibility sizes) */
  K=mxGetM(prhs[5]);
  if(K!=N) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetN(prhs[5]);
  if(K!=1) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetM(prhs[6]);
  if(K!=N) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  K=mxGetN(prhs[6]);
  if(K!=J) 
    mexErrMsgIdAndTxt("Dalg:InputSize","Error in input sizes!");
  
  /* Get all the pointers */
  u=mxGetPr(prhs[0]);
  v=mxGetPr(prhs[1]);
  qty=mxGetPr(prhs[2]);
  qty_pl=mxGetPr(prhs[3]);
  qty_fl=mxGetPr(prhs[4]);
  stu_pl_ty=mxGetLogicals(prhs[5]);
  elig=mxGetLogicals(prhs[6]);

  /* Allocate an application stack - this is the output when there are no rejections */
  plhs[0]=mxCreateDoubleMatrix(N,1,mxREAL);
  plhs[1]=mxCreateDoubleMatrix(J,1,mxREAL);
  plhs[2]=mxCreateDoubleMatrix(J,1,mxREAL);
  app_pointer=mxGetPr(plhs[0]);
  cutoff_pl=mxGetPr(plhs[1]);
  cutoff_fl=mxGetPr(plhs[2]);

  for(i=0;i<N;i++)
    {
      app_pointer[i]=0;
    }

  for(j=0;j<J;j++)
    {
      cutoff_pl[j]=0;
      cutoff_fl[j]=0;
    }

  /* Allocate memory */
  rejected=mxCalloc(N,sizeof(bool));

  /* Collect a sort index on both sides */
  u_sort=mxCalloc(J*N,sizeof(double));
  v_sort=mxCalloc(N*J,sizeof(double));

  for(i=0;i<N;i++)
    {
      sort_u_Array(&u_sort[i*J],&u[i*J],J);
    }
  for(i=0;i<J;i++)
    {
      sort_u_Array(&v_sort[i*N],&v[i*N],N);
    }

  do {
    any_reject=false;
    /* Go through each firm, and keep the best applications */
    for(j=0;j<J;j++)
    {
        /* Init app_counts to 0 */
	app_count=0;
	app_pl_count=0;
	app_fl_count=0;	

	/* Initialize cutoffs to above the highest possible value */ 
	cutoff_pl[j]=MAXDUB;
	cutoff_fl[j]=MAXDUB;

	/* Consider applicants in descending order */
	for(i=0;i<N;i++)
        {
            appl_id=(int) v_sort[j*N+i];
            if(u_sort[appl_id*J+((int) app_pointer[appl_id])]==j)
	      {
		if(!elig[appl_id+j*N])
		  {
		    rejected[appl_id]=true;
		    any_reject=true;		    
		  }
		if(elig[appl_id+j*N])
		  {
		    /* Increment appropriate app count */                       
		    if(stu_pl_ty[appl_id])
		      {
			if(app_count<((int) qty[j]) && app_pl_count<((int) qty_pl[j]))
			  {   
			    app_pl_count++;
			    app_count++;
			    cutoff_pl[j]=v[j*N+appl_id];
			  }
			else
			  {
			    rejected[appl_id]=true;
			    any_reject=true;
			  }
		      }
		    else 
		      {
			if(app_count<((int) qty[j]) && app_fl_count<((int) qty_fl[j]))
			  {   
			    app_fl_count++;
			    app_count++;
			    cutoff_fl[j]=v[j*N+appl_id];
			  }
			else
			  {
			    rejected[appl_id]=true;
			    any_reject=true;
			  }
		      } 
		  }
	      }
        }

	/* Overwrite cutoff if there's spare capacity */

	/* School cutoff is the minimum of the two cutoffs  */
	cutoff_sch=cutoff_pl[j]<cutoff_fl[j]?cutoff_pl[j]:cutoff_fl[j];

	if (app_pl_count<((int) qty_pl[j]) && app_count<((int) qty[j]))
	  cutoff_pl[j] = 0;
	if (app_pl_count<((int) qty_pl[j]) && app_count==((int) qty[j]))
	  cutoff_pl[j] = cutoff_sch; 

	if (app_fl_count<((int) qty_fl[j]) && app_count<((int) qty[j]))
	  cutoff_fl[j] = 0;
	if (app_fl_count<((int) qty_fl[j]) && app_count==((int) qty[j]))
	  cutoff_fl[j] = cutoff_sch;  
    }
    
    /* Carry through the rejections */
    for(i=0;i<N;i++)
        {
            if(rejected[i])
                app_pointer[i]++;	      

            rejected[i]=false;
        }
    } while(any_reject);


  for(i=0;i<N;i++)
    {
      app_pointer[i]=u_sort[i*J+((int) app_pointer[i])]+1;
    }

  /* Free up memory */
  mxFree(v_sort);
  mxFree(u_sort);
  mxFree(rejected);
}
