addpath ../estimation;
load('../estimation/data.mat');

% Set defaults
if ~exist('do_overwrite','var'), do_overwrite=true; end
if ~exist('truthful_behavior','var'), truthful_behavior=false; end
if ~exist('smpl','var'), smpl=bh.smpl; end	
if ~exist('save_disk_space','var'), save_disk_space=false; end

%% Set Parameters and filename
num_sim = do_value_sim;
 
%% Read in master file
load([folder '/master']);

hacked_distances = distances; hacked_distances(isnan(distances))=1;
mean_utility=reshape(X*B,size(distances))-hacked_distances;
years=unique(year(esample))';  % year(esample) avoids running years with no lotteries in lastyear.

n_prog=size(consolidate.map,1);

%% What to do?
do.cambridge_welfare = do_overwrite || ~exist('V_camb','var') || ~exist('camb_assign','var');
do.justified_envy    = do_overwrite || ~exist('JE_camb');
do.DA_cutoffs        = do_overwrite || ~exist('cutoffs','var');
do.DA_welfare        = do_overwrite || ~exist('V_da','var') || ~exist('da_assign','var');
if save_disk_space
    do.DA_welfare        = do.DA_welfare && (do_overwrite || ~exist('pref_da_camb','var') || ~exist('frac_pref_da','var') || ~exist('frac_indiff','var'));
    do.DA_cutoffs        = do.DA_cutoffs && do.DA_welfare;
    do.cambridge_welfare = do.cambridge_welfare && (do.DA_welfare || do.justified_envy);
end

if do.DA_cutoffs
    cutoffs.pl=zeros(length(years),num_sim,n_prog);
    cutoffs.fl=zeros(length(years),num_sim,n_prog);
end

%% Start
for yy=1:numel(years)

    counter_year=years(yy);
    fprintf('Running Year %d\n',counter_year)
    
    % Which set of students? Can restrict to rationalized or ok students
    student_set=smpl(year(smpl)==counter_year);

    % Number of students
    n_stu_counter=numel(student_set);

    % Mean utilities at B
    mu=mean_utility(student_set,:); % Generate mu
    
    % Priorities
    priority_mat=prxd.data(student_set,:) + 2*sibl.data(student_set,:);
    paidlunch_counter=paidlunch(student_set);

    % Checks
    assert(size(priority_mat,1)==n_stu_counter);  % Ensure size compatibility
    
    % Eligibility
    elig_counter=full(1-ineg.data(student_set,:));    

    %%%%%%%%% CAMB WELFARE  %%%%%%%%%%%
    if do.cambridge_welfare,
    fprintf('Computing Welfare under Cambridge ...')
    chosen_list_yy=zeros(n_stu_counter,3,num_sim);
    camb_assign_yy=zeros(n_stu_counter,3,num_sim);
    V_camb_yy=zeros(n_stu_counter,num_sim);
    V_vec_yy=zeros(n_stu_counter,n_prog,num_sim);
    bh.counter_stu_type=bh.student_type(student_set);
    as.counter_stu_type=as.student_type(student_set); 

    parfor ss=1:num_sim
        % Simulate over V draws for each ii to expected utility        
         rng(ss,'CombRecursive');   % y default, the MATLAB® client and MATLAB workers use different random number generators, even if the workers are part of a local cluster on the same machine with the client. For the client, the default is the Mersenne Twister generator ('twister'), and for the workers the default is the Combined Multiple Recursive generator ('CombRecursive' or 'mrg32k3a'). 
         v_ss=mvnrnd(mu,S);
         for ii = 1:n_stu_counter % ii indexes individuals in the bootstrap sample
            v_ii = v_ss(ii,:);
            bhl = bh.lott(bh.counter_stu_type(ii));
            asl = as.lott(as.counter_stu_type(ii));
            % Choose a p - get indirect utility of top choice given behavioral assumption.
            if truthful_behavior;
                all_lot=asl.choice;
                all_lot(isnan(all_lot) | all_lot==0)=n_prog+1; % replace reference to the outside option
                v_iic = [v_ii(consolidate.map) 0]; 
                iu = all_lot; iu(:) = v_iic(all_lot(:)); % Utility implied by each program ranked
                [~,index] = sortrows(iu); % if indifferent behave truthfully
                chosen_lott = index(end);
                chosen_list_yy(ii,:,ss)=asl.choice(chosen_lott,:);     
            else
                [~,chosen_lott]=max(bhl.p*v_ii(consolidate.map)');
                a0 =asl.choice(chosen_lott,:); a0(isnan(a0))=0; 
                a1 = bhl.choice(chosen_lott,:); a1(isnan(a1))=0; assert(all(a0==a1));
                % This assertion verifies that the assignment and behavioral lotteries sort choices identically.
                chosen_list_yy(ii,:,ss)=bhl.choice(chosen_lott,:);     
            end

            % Put down the chosen probabilities
            [~,where_in_p]=ismember(chosen_list_yy(ii,:,ss),1:n_prog);
            where_in_p=where_in_p(where_in_p~=0);  % Remove unranked
            p_temp=zeros(1,3); % temporary vector - to get around parfor restrictions
            p_temp(1:size(where_in_p,2))=asl.p(chosen_lott,where_in_p);
            camb_assign_yy(ii,:,ss)=p_temp;        

            % Store the utilities
            V_vec_yy(ii,:,ss)=v_ii(consolidate.map);            
            V_camb_yy(ii,ss) = asl.p(chosen_lott,:)*v_ii(consolidate.map)';
            
            % Justified Envy
            
        end
    
    end
    V_vec{yy}=V_vec_yy;
    V_camb{yy}=V_camb_yy;
    camb_assign{yy}=camb_assign_yy;
    chosen_list{yy}=chosen_list_yy;
    fprintf('Done\n')
    end % Done computing Welfare under Cambridge
    
    %%%%%%%%% CAMBRIDGE JUSTIFIED ENVY ********
    if do.justified_envy
    fprintf('Running JE... ');
    V_vec_yy = V_vec{yy};
    chosen_list_yy = chosen_list{yy};
    JE_camb_yy = zeros(n_stu_counter,num_sim);
    cutoffs_yy = squeeze(cutoffs_camb(yy,:,:));
    parfor ii=1:n_stu_counter
        v_ii = squeeze(V_vec_yy(ii,:,:))';
        choices = squeeze(chosen_list_yy(ii,:,:))';
        JE_camb_yy(ii,:)  = camb_mech_points(choices,paidlunch_counter(ii,:),priority_mat(ii,:),cutoffs_yy,v_ii);
    end
    JE_camb{yy} = JE_camb_yy;
    frac_je{yy} = mean(JE_camb_yy,2);
    fprintf('Done\n')
    end
    
    %%%%%%%%% DA CUTOFFS %%%%%%%%%%%
    if do.DA_cutoffs,

    fprintf('Running DA_cutoffs... ')
    % Quantities
    capacities=kpct.data(:,:,kpct.year==counter_year);

    qty.sch=capacities(:,3);
    qty.pl=capacities(:,1);
    qty.fl=capacities(:,2);

    % Add outside option
    qty.sch=[qty.sch;n_stu_counter];
    qty.pl=[qty.pl;n_stu_counter];
    qty.fl=[qty.fl;n_stu_counter];
    priority_mat=[priority_mat zeros(n_stu_counter,1)];

    % Allocate memory
    p_pl = zeros(num_sim,n_prog+1);
    p_fl = zeros(num_sim,n_prog+1);
    
    parfor bb=1:num_sim
        rnd_seed = bb + 1000;
        [p] = compute_DA_cutoffs(mu,S,priority_mat,elig_counter,paidlunch_counter,qty,consolidate,rnd_seed);
        p_pl(bb,:)=p.pl;
        p_fl(bb,:)=p.fl;
    end
    cutoffs.pl(yy,:,:)=p_pl(:,1:end-1);
    cutoffs.fl(yy,:,:)=p_fl(:,1:end-1);
    priority_mat=priority_mat(:,1:end-1);
    fprintf('Done\n')
    end % Done running DA_cutoffs
    
    
    %%%%%%%%% DA WELFARE %%%%%%%%%%%
    if do.DA_welfare
    fprintf('Computing Welfare under DA ...')
    da_assign_yy=zeros(n_stu_counter,size(mu,2),num_sim);
    V_da_yy=zeros(n_stu_counter,num_sim);
 
    parfor ss=1:num_sim
        rng(ss,'CombRecursive');
        v_ss=mvnrnd(mu,S);
        for ii = 1:n_stu_counter % ii indexes individuals in the bootstrap sample
            v_ii = v_ss(ii,:);
            if paidlunch_counter(ii), p = squeeze(cutoffs.pl(yy,:,:)); else p = squeeze(cutoffs.fl(yy,:,:)); end
            f = priority_mat(ii,:);
            % Simulate over V draws for each ii to expected utility        
            
            da_assign_ss = zeros(size(v_ii)); % da_assign_ss has the same dimension as the number of programs
            V_da = 0; % da_assign_ss has the same dimension as the number of programs
            [sorted_v_ii,ROL] = sort(v_ii,'descend');
            acc_asgn_prob = 0;     % will accumulate assignment probability
            for r = 1:numel(ROL)   % iterate over each position in the waitlist
                if sorted_v_ii(r)>0 & min(acc_asgn_prob)<1
                    which_school = ROL(r);   % which school was ranked in the r position (1-13)?
                    which_programs = (elig_counter(ii,:)==1) & (consolidate.map'==which_school); % which programs from this school is this student eligible for (1-16)?           
                    assert(sum(which_programs)>0);  
                    min_draw_needed = min(bsxfun(@minus,p(:,which_programs),f(which_programs)),[],2);  % min lottery draw needed [>1 :  no chance]; [(0,1) with a good lottery draw]; [<0 with any lottery draw];
                    min_draw_needed = max(min_draw_needed,0);
                    new_asgn_prob = max(1 - acc_asgn_prob- min_draw_needed,0); 
                    da_assign_ss(which_school) = da_assign_ss(which_school) + mean(new_asgn_prob); % Average across cutoff draws
                    acc_asgn_prob = acc_asgn_prob + new_asgn_prob;    % Lottery draws that cleared a cutoff up to rank r.
                end
            end
            da_assign_yy(ii,:,ss) = da_assign_ss';
            V_da_yy(ii,ss) = (da_assign_ss*v_ii');
            
        end
    end 
    V_da{yy}=V_da_yy;
    da_assign{yy}=da_assign_yy;
    fprintf('Done\n')
    end % Done computing Welfare under DA
    
    
end   % Loop over years


if ~save_disk_space
    fprintf('Saving a large file ...')
    if do.DA_cutoffs && do.DA_welfare && do.cambridge_welfare && do.justified_envy
        save(savefname,'cutoffs','V_vec', 'camb_assign', 'da_assign', 'V_da', 'V_camb', 'chosen_list','JE_camb');
    else
        if do.DA_cutoffs, save(savefname,'cutoffs','-append'); end
        if do.DA_welfare, save(savefname,'V_da','da_assign','-append'); end
        if do.cambridge_welfare, save(savefname,'V_vec', 'camb_assign', 'V_camb','chosen_list','-append'); end
        if do.justified_envy, save(savefname,'JE_camb','-append'); end
    end
    fprintf('Done\n')
else 
    if ~exist('pref_da_camb','var') ||  ~exist('frac_pref_da','var') ||  ~exist('frac_indiff','var')
        pref_da_camb=cell(numel(V_da),1);
        for yy = 1:numel(V_da)
            pref_da_camb{yy}=mean(V_da{yy}-V_camb{yy},2);
            frac_pref_da{yy}=mean(V_da{yy}-V_camb{yy}>1e-5,2);
            frac_indiff{yy}=mean(abs(V_da{yy}-V_camb{yy})<=1e-5,2);
        end
    end
    
    fprintf('Saving a small file ...')
    save(savefname,'pref_da_camb','frac_pref_da','frac_indiff','frac_je');
    fprintf('Done\n')
end    
