%%% Creates a constraint matrix from the lottery file and figures out an
%%% initial value in the cone

% clearvars -except method
%% Set parameters

%%method='last_year_ts' % the method here to fullinfo,
                          % current_year, or current_year_av, last_year
my_eps=1e-6;  

%% Script
load ../estimation/data;

% Initialize matrices
constraint_matrix=cell(size(student_id,1),1);
tag = cell(size(constraint_matrix));
rationalized_students=true(size(student_id));
V=zeros(size(student_id,1),size(prox.data,2));

if ~exist('smpl','var'), smpl=1:size(student_id,1); end

% Create constraint and find init value
for ss=1:size(rank.data,1)
    s = smpl(ss);
    if mod(s,100)==0
       disp(['Student number ' num2str(s) ' of ' num2str(size(rank.data,1))]); 
    end
    
    % Reshape the ranks
    [~,ranks]=ismember([1,2,3]',rank.data(s,:));    
        
    % Figure out location of chosen lottery
    if ismember(method,{'current_year','current_year_av','last_year','current_year_ts','current_year_av_ts','current_year_av_ts_full','last_year_ts','current_year_ts_wl','current_year_ts3'})
        stu_loc=student_type(s);
    elseif isequal(method,'fullinfo')
        stu_loc=s;
    else
        disp('Unrecognized method: type dbcont to continue using standard routine	');
		stu_loc=student_type(s);
    end
    % Need to replace NaN to 0
    choice_set=lott(stu_loc).choice;
    choice_set(isnan(choice_set))=0;
    if ~isempty(choice_set)
        [check,chosen_lott]=ismember(ranks',choice_set,'rows');

        if ~check
           disp(['Whoops! check out student ' num2str(s)]);
           pause;
        end
        
        % Compute the difference
        chosen_p=lott(stu_loc).p(chosen_lott,:);
        duplicate_lotteries=ismemberf(full(lott(stu_loc).p),full(chosen_p),'rows','tol',1e-4);
        con_mat=repmat(chosen_p,sum(~duplicate_lotteries),1)-lott(stu_loc).p(~duplicate_lotteries,:);
            
        % Solve for con_mat*v>0
        model=struct();
        model.A=sparse(con_mat);
        model.sense='>';
        model.lb=repmat(-Inf,size(con_mat,2),1);   % Gurobi defaults to a lower bound of 0 - need to allow for outside option to be worse
        model.rhs=my_eps*ones(size(con_mat,1),1);
        model.obj=zeros(size(con_mat,2),1);
        
        params = struct();
        params.Presolve = 0;
        params.outputflag = 0;
        params.FeasibilityTol = 1e-9;

        result = gurobi(model, params);

        % Store whether the student was rationalized or not, and if
        % (s)he was, what is the init utility
        if strcmp(result.status, 'INFEASIBLE')==1
            rationalized_students(ss)=false;        
        else
            V(ss,:)=result.x(:);
            assert(all(con_mat*V(ss,:)' >0));
        end
        
        % Set constraint_matrix
        constraint_matrix{ss}=sparse(con_mat);
        tag{ss}=duplicate_lotteries;
    end
end

disp(['Fraction of rationalized students: ' num2str(mean(rationalized_students))]);
