
%% Generates the constraints files
addpath ../boston_code
addpath ../estimation

if ~exist('do_draw_num','var'), do_draw_num= 0; end % default if do_draws doesn't exist
if ~exist('do_overwrite','var'), do_overwrite= 0; end
if ~exist('do_specifications','var'), 
    do_specifications = {'current_year','last_year','current_year_av','current_year_wl'};  
end 
if ~exist('spec_subset','var'), 
    spec_subset = do_specifications;  
end 
spec_subset = ismember(do_specifications,spec_subset);

for do_spec = do_specifications(spec_subset)
    clearvars -except do_* tasks folder
    do_spec = do_spec{1};
    switch do_spec
        case 'current_year',     method = 'current_year_ts';    do_constraints = (do_draw_num ==0);
        case 'current_year_av',  method = 'current_year_av_ts'; do_constraints = (do_draw_num ==0);
        case 'current_year_wl',  method = 'current_year_ts_wl'; do_constraints = (do_draw_num ==0);
        case 'last_year',        method = 'last_year_ts';       do_constraints = (do_draw_num ==0);
        case 'mixture',          method = 'current_year_ts';    do_constraints = false; 
        case 'truthful',         method = 'truthful';           do_constraints = false;
        otherwise, fprintf('Unrecognized method\n'); keyboard;
    end

    % String version of the draw number and pick bootstrap file/subfolder
    if do_draw_num==0, 
        draw_s = ''; subfolder = ''; cutoffs_file = 'cutoffs_ts_bootstrap';    % The release version uses the same file for estimation and bootstrap. 
        if ~do_constraints, continue; end    % No need to compute the constraint matrix
    else                                                                       
        draw_s = num2str(do_draw_num); subfolder = 'bootstrap/'; cutoffs_file = 'cutoffs_ts_bootstrap';
    end
    
    % Generate lott structure (assignment probabilities for each choice)
    % Base lott (does it exist?)
    base_lott = fullfile(cd, [subfolder 'current_year_ts' draw_s '.mat']);
    if ~strcmp(method,'last_year_ts') % last_year_ts doesn't need the base file
        % Create it if there is a need to overwrite or if it doesn't exist:
        if (strcmp(method,'current_year_ts') && do_overwrite==1) || ~(exist(base_lott, 'file')==2),
            [lott,smpl,student_type]=gen_lott_file(cutoffs_file,do_draw_num,0); % create it
        else
            load(base_lott,'lott','student_type','smpl');   % load it
        end
    end

    % Method specific lott
    switch method
        case 'current_year_av_ts'
            load('data','year');
            current_year_av_ts; 
        case 'current_year_ts_wl'
            load('data','asgn','rank','year','regs','paidlunch');
            current_year_ts_wl;
        case 'last_year_ts'
            [lott,smpl,student_type]=gen_lott_file(cutoffs_file,do_draw_num,1);
    end
    
    if do_constraints,        constraints;        end   % Generate constraint matrix

    % Save a select set of files: current_year_ts or baseline files for alternative methods.
    if do_overwrite==1 || ~exist([subfolder method draw_s '.mat'], 'file')
        saveFile = [subfolder method draw_s];
        if do_constraints
            save(saveFile,'lott','student_type','constraint_matrix','V','rationalized_students','smpl','tag');
            fprintf('Saved %s (large)\n',saveFile);
        elseif strcmp(do_spec,'current_year')
            save(saveFile,'lott','student_type','smpl');
            fprintf('Saved %s (small)\n',saveFile);
        end
    end
end

