%% Init
clear -tasks;
cutoffs_file = 'cutoffs_ts_bootstrap';

%matlabpool close force;
%matlabpool open 100;

addpath ../estimation;
addpath ../boston_code;

load data;

NPROG=size(prox.data,2);
% Bootstrap/Subsampling samples; 
% b = fraction of students to be sampled (1 for bootstrap <1 for subsampling)
NSUB = 1000; b = 1; 
% Number of draws within each bootstrap/subsample draw 
NSIM = 1000;
NLOT = 1;


%% Prepare for run

% Get students with unique types - by year, paidlunch, priority (as calculated) and eligibility
[unique_types,which_rows,student_type]=unique([year paidlunch prxd.data sibl.data ineg.data],'rows');
priorities=prxd.data+2*sibl.data;

% Reshape rank data: one row per student, 3 columns with the indices of schools ranked 1st, 2nd & 3rd)
rank_reshape=zeros(size(size(rank.data,1),1),3);
for s=1:size(rank.data,1)
    [~,rank_reshape(s,:)]=ismember([1,2,3]',rank.data(s,:));
end
rank_reshape(rank_reshape==0)=NaN;

%% Simulate Samples
n_students = numel(paidlunch);
styp=struct();
draw=zeros(max(year)-min(year)+1,NSUB,NSIM,NLOT,2*NPROG);
estm=zeros(max(year)-min(year)+1,NSIM,NLOT,2*NPROG);
bsmp=zeros(NSUB,n_students);
for uu=1:size(unique_types,1)
    % Find a student of this type    
    type_row=which_rows(uu);
    % Compute possible rankings   
    type_options=find(ineg.data(type_row,:)==0);
    type_n_options=length(type_options);
    choices =type_options(combinator(type_n_options,3,'p'));
    keep_lott=find(~any(choices==3,2) | ~any(choices==4,2)); 
    styp(uu).choice =choices(keep_lott,:);
    styp(uu).paidlunch = paidlunch(which_rows(uu));
    styp(uu).priorities = priorities(which_rows(uu),:);
    
end                
rng(100); % Be certain that the other students faced by all students are the same across ranks
tot_students = 0;
for tt=min(year):max(year)
    disp(['Calculating Year ' num2str(tt)]);
    stu_year_rows=find(year==tt);   
    n_stu_year=length(stu_year_rows);
    capacities=kpct.data(:,:,kpct.year==tt);
    types_year = student_type(stu_year_rows);
    % m_hat_year = hist(types_year,1:max(student_type));
    tic
    
    for bb=0:NSUB
        
		if bb>0
			if b<1
				% Without replacement;
				rand_per = randper(n_stu_year); % Draw b*n students
				rand_stu = rand_per(1:ceil(b*n_stu_year));
				else
				% With replacement;
				rand_stu=ceil(rand(n_stu_year,1)*n_stu_year); % Draw n students
				bsmp(bb,tot_students+1:tot_students+n_stu_year) = stu_year_rows(rand_stu);
			end
		else
			rand_stu = 1:n_stu_year; % Use the actual data for bb=0;
		end
			

        for ss=1:NSIM
			% Draw (with replacement) a set of other students from rand_stu
			rand_stu_ss=ceil(rand(n_stu_year-1,1)*numel(rand_stu)); % Draw n-1 other students
			sim_stu=stu_year_rows(rand_stu(rand_stu_ss));
            %m_hat_b(:,ss) = hist(student_type(sim_stu),1:max(student_type));
            % Collect inputs for camb_mech_sch_c.
            % First student is focal student, remaining students are
            % bootstrapped
            sim_rank_list=rank_reshape(sim_stu,:); 

            sim_paidlunch=paidlunch(sim_stu);
            sim_prox= prox.data(sim_stu,:);  % Type proximity using calculated distances and sample proximity using typerted priorities
            sim_sibl=sibl.data(sim_stu,:);  % Sibling priority

            % Collect the priorities and the lotteries
            sim_priorities  = full(priorities(sim_stu,:));
            
            for ll = 1:NLOT
                sim_lotteries=rand(n_stu_year-1,1);

            % Check that the inputs are the right data format
                assert(~issparse(capacities));
                assert(~issparse(sim_priorities));
                assert(~islogical(sim_paidlunch));

            % Collect the assignment over the first student
                points_pivot=camb_mech_points(sim_rank_list, sim_paidlunch, capacities(:,3), capacities(:,1), capacities(:,2), sim_priorities, sim_lotteries);
                
			if bb==0
				estm(tt-min(year)+1,ss,ll,:)=points_pivot;
			else
			    draw(tt-min(year)+1,bb,ss,ll,:)=points_pivot;
			end
            end
        end
       fprintf('.')
       if mod(bb,100)==0, fprintf('\n'); toc; tic; end
    end
	tot_students=tot_students+n_stu_year;
end
toc
pause(5);
save(cutoffs_file,'student_type','unique_types','styp','draw','b','estm','bsmp','-v7.3');
toc;


 
