
%% Generates the constraints files

if ~exist('do_draws','var'), do_draws = 0:1000; end % default if run_draws doesn't exist
if ~exist('do_specifications','var'), 
    do_specifications = {'current_year','last_year','current_year_av','current_year_wl','mixture'};  
end 

clearvars -except do_* tasks;

% Computational options
delete(gcp('nocreate'));
parpool('local',16);

% Bootstrap: Version control
do_version = 0;
do_version_accepted = -1; 
do_gibbs = '';
do_stem = ['param_A_']; do_pick = do_stem;
do_NITER =100000; do_burn_in = 50000; do_per_run = 500;
% do_overwrite: (1) disregard previous results; (0) only execute if file doesn't exist
%               (-1) execute if file doesn't exist of it is not complete;
%               (-2) only execute if the file exist and is incomplete, i.e. completes the file
% finding a file with and earlier version than accepted is equivalent to not finding the file.

for do_draw_num = do_draws
    if do_draw_num == 0, continue; end
    for do_spec = do_specifications
        clearvars -except do_* tasks
        do_spec = do_spec{1};
        
        % folder for results
        
        switch do_spec
            case 'current_year',     folder = '../estimation/currentyear1_ts';
            case 'current_year_av',  folder = '../estimation/currentyearav1_ts';
            case 'current_year_wl',  folder = '../estimation/currentyear1_ts_wl';
            case 'last_year',        folder = '../estimation/lastyear1_ts';
            case 'mixture',          folder = '../estimation/mixture1_ts';
            case 'truthful', continue;
            otherwise, fprintf('Unrecognized method\n'); keyboard;
        end
        
        % First figure out if we need to execute this part of the code;
        fprintf('%s draw %d\n',do_spec,do_draw_num);
        if do_overwrite < 1
            this_file_name = [do_pick num2str(do_draw_num) '.mat'];
            found_file = exist(fullfile(folder, this_file_name), 'file')==2;
            if found_file
                old_file = load(fullfile(folder, this_file_name));
                found_file = found_file && isfield(old_file.final_vals,'version') && old_file.final_vals.version>=do_version_accepted;
                if ~found_file, clear('old_file'); end
            end
            if  found_file && do_overwrite == 0, continue; end % skip the rest of the code
            if ~found_file && do_overwrite == -2, continue; end % skip the rest of the code
        end
        spec_subset = {do_spec};
        gen_constraints_ts; % generates the constraints matrix

        shr = 0.001*(do_draw_num>0); shx = 0; % To avoid zero probabilities in bootstrap samples
        % shr = weight on the observed L, shx = weight on the wishful lottery: 1/2 1st rank, 1/3 2nd rank and 1/6 3rd rank.
        run_bootstrap_chain;
        
        
    end
end

% Read the tables once more to add the bootstrap SE results to the summary file.
do = struct('read_chain',1); cd('../estimation'); run_estimation; 

delete(gcp('nocreate'));
