function [meanx,sigma2_n]=btch_sig(vec,b,dim)
% function [meanx,sigma2_n]=btch_sig(vec,b,dim)
%
% This function obtains an estimate for the variance of a MCMC using
% nonoverlaping batch means. 
% Output:
% meanx: overall mean, which is an estimate of \mu;
% sigma2_n: estimate of the variance of (meanx-\mu) for the whole chain.
% Input:
% vec: vector/matrix/array with the MCMC.
% b: batch size
% dim: computes the mean and variance along this dimension. Default if omitted: 1.
% Please, input vec,b and dim such that size(vec,dim)/b is an integer.

% References: http://www.mcmchandbook.net/HandbookChapter1.pdf (page 11).
% Paulo Somaini.


if nargin<3
    [meanx,sigma2_n]=btch_one(vec,b);
else % It is only important if dim ~=1;
    szarr =size(vec);
    order = 1:numel(szarr);
    order(dim)=[];
    neworder=[dim order];
    vec = permute(vec,neworder);
    [~,iy]=ind2sub(size(vec),1:numel(vec));
    szarr(dim)=1;
    meanx = zeros(szarr);
    sigma_n =zeros(szarr);
    for y =1:max(iy)
        v = vec(:);
        v = v(iy==y);
        [my,sy] = btch_one(v,b);
        meanx(y)=my;
        sigma2_n(y)=sy;
    end
end

function [meanx,sigma2_n]=btch_one(vec,b)

n=numel(vec);
m=n/b;
meanx = mean(vec);
mat=reshape(vec,[b,m]);
mbk=mean(mat,1)-meanx;
sigma2_b = mean(mbk.^2);
sigma2_n = b/n*sigma2_b;

end

end