
% % This script creates the design matrix from data.mat. It requires the
% % variables language, other_cov_list, eth_list and lang_list, consolidate.
% language = 'hlan'; % hlan = language at home, slan = primary spoken language
% prox_dummies = 'prxd','prox','none';
% % Choose which ethnicities and languages to include (the first one is the no-dummy; everything else is "other")
% other_cov_list = {'Constant', 'Paid Lunch', 'Sibling'}; % Constant, Paid Lunch, Sibling
% eth_list = {'White', 'Black', 'Asian', 'Hispanic'}; % White, Black, Asian, Hispanic
% lang_list = {'English', 'Spanish', 'Portuguese'}; % English, Spanish, Portuguese, Creole
% consolidate.name = {'Baldwin','Haggerty',{'Amigos','Amigos Bilingual'}, 'MLK',...
%     'King Open','Morse','Peabody',{'Tobin','Tobin K4','Tobin K5'},'Flet Mayn',...
%     'Graham Parks', 'Kenn Long','Cambridgeport','King Open Ola'};
%% Load Data (may be loaded before) 
load data
fprintf('data.mat loaded \n');
ns = numel(student_id);


%% Ethnicity
[isit,where]=ismember(eth_list,ethn.name);
if any(~isit), error(['Ethnicity(ies) ' eth_list{~isit} ' not found.']); end
eth_X.name = cat(2,ethn.name(where(2:end)),'Other Ethn');
eth_X.data = [ethn.data(:,where(2:end)) sum(ethn.data,2)-sum(ethn.data(:,where),2)];

%% Language;
eval(['lang = ' language(1:4) ';']);
[isit,where]=ismember(lang_list,lang.name);
if any(~isit), error(['Language(s) ' lang_list{~isit} ' not found.']); end
lang_X.name = cat(2,lang.name(where(2:end)),'Other Lang');
lang_X.data = [lang.data(:,where(2:end)) sum(lang.data,2)-sum(lang.data(:,where),2)];

%% Other Covariates
covs_X.name = []; 
covs_X.data = [];
for cn = 1:numel(other_cov_list)
    switch other_cov_list{cn}
        case 'Constant'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = ones(ns,1);
        case 'Paid Lunch'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = paidlunch;
        case 'Age'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = elig_age;
        case 'Household Income'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = med_hhold_inc;
        case 'Family Income'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = med_fam_inc;
        case 'Sibling'
            covs_X.name{cn} = other_cov_list{cn};
            covs_X.data(:,cn) = zeros(ns,1); %Placeholder for sibling
        otherwise
            error(['Covariate ' other_cov_list{cn} ' not found.']);
    end
end

if exist('prox_dummies','var')
	if exist(prox_dummies,'var') && ~strcmp(prox_dummies,'none') 
		wprx = eval(prox_dummies);
		covs_X.name{cn+1} = 'Proximity';
		covs_X.data(:,cn+1) = zeros(ns,1); %Placeholder for proximity
	end
end

    
    

Xmat.name = cat(2,covs_X.name,eth_X.name,lang_X.name);
Xmat.data = [covs_X.data eth_X.data lang_X.data];

%% Build X using consolidate;
n_progs = numel(consolidate.name);
Xcell = cell(1, n_progs);
consolidate.map = zeros(numel(prox.name),1);
distances=zeros(ns,n_progs);
for pr=1:n_progs
    [isit,where]=ismember(consolidate.name{pr},prox.name);
    consolidate.map(where)=pr;
    if any(~isit), error(['Program ' lang_list{~isit} ' not found.']); end
    distances(:,pr) = min(dists(:,where),[],2);
    if exist('wprx','var'), this_prox = max(wprx.data(:,where),[],2); end
    this_sibl = max(sibl.data(:,where),[],2);
    this_mat = Xmat.data;
    [isit,where]=ismember('Sibling',Xmat.name);
    if isit, this_mat(:,where) = this_sibl; end 
    [isit,where]=ismember('Proximity',Xmat.name);
    if isit, this_mat(:,where) = this_prox; end
    Xcell{pr} = this_mat;
end
    

% This script produced:
% consolidate.map;
% distances;
% Xcell;