function [ final_vals, priors, run_next_list] = gen_chain_mcmc_mex_mix(folder,priors,init_vals,chain,run_this_list,NITER_by_file,stem,thin_each,groups)
%GEN_CHAIN_MCMC Generates a MCMC chain using the gibbs sampler.
%   It loads the master.mat file in 'folder', and the 'priors' and
%   'init_vals' to run an MCMC using gibbs_sampler_test_sur_v. It splits
%   the chain in files [stem chain j.mat] for j in run_this_list.
%   The main output is the stored file, but it also returns the variables
%   that are useful to continue the chain.

if nargin<8, thin_each=1; end % Do not thin file if thin_each is not included.
if nargin<9, groups=[]; end % Do not thin file if thin_each is not included.

% Load master file and transform Xcell in X if needed.
X=[];
load([folder '/master.mat']);

% Construct X, if necessary.
if isempty(X), X = blkdiag(Xcell{:}); end
n_coeff = size(X, 2);
n_cmat = numel(V);
constraint_matrix=constraint_matrix(esample,:);
distances = distances(esample,:);
for cm = 1:n_cmat, V{cm}=V{cm}(esample,:); end
X=X(repmat(esample(:),n_progs,1),:);



% Diagnose Errors
assert(sum(sum(isnan(X))) + sum(sum(isinf(X))) == 0); %infs or nans in X
assert(sum(sum(isnan(distances))) + sum(sum(isinf(distances))) == 0); %infs or nans in distances
assert(sprank(X)==min(size(X))); % Full rank design matrix;


% Set deault values if not provided.
if nargin<2 ||isempty(priors),
    priors.b0 = zeros(n_coeff, 1);
    priors.var0 = 100 * eye(n_coeff);
    priors.nu = 100;
    priors.V_wishart = eye(n_progs);
    priors.alpha = 1; priors.beta = 1;
end
if nargin<3 || isempty(init_vals), 
    init_vals.N = mod(1:sum(esample),n_cmat)+1;
    %for cm = 1:n_cmat, init_vals.V(init_vals.N==cm,:)=V{cm}(init_vals.N==cm,:); end
    init_vals.V=V{2};
end
if nargin<4 || isempty(chain) , chain=0; end
if nargin<5 || isempty(run_this_list), run_this_list=[]; end
if nargin<6 || isempty(NITER_by_file), NITER_by_file=1000; end
if nargin<7 || isempty(stem), stem='chain_'; end

% Resume option: if init_vals is an integer, then load 
% [folder '/' stem num2str(chain) '_run_' num2str(init_vals)] 
if isreal(init_vals) && numel(init_vals)==1 && mod(init_vals,1)==0 % If we are resuming.
    resumeAt= init_vals; run_this_new_list = run_this_list(run_this_list>resumeAt);
    load([folder '/' stem num2str(chain) '_run_' num2str(resumeAt)], 'final_vals'); init_vals=final_vals;
    [final_vals, priors] = gen_chain_mcmc_mex_mix(folder,priors,init_vals,chain,run_this_new_list,NITER_by_file,stem,thin_each,groups);
    
else % If we are not resuming
    % Run the MCMC using the Gibbs sampler.
    final_vals = init_vals;

    if numel(run_this_list)>0
        fprintf('Ready to run chain %d!\n',chain);
    else
        fprintf('Returning initial values and  default priors\n');
    end    

    for savenum = run_this_list;
        tic; [all_B_vec, all_Sigma, all_V, all_N, all_Ng, final_vals,init_vals] = gibbsSamplerMixMex(X, constraint_matrix, priors, [], distances, NITER_by_file, final_vals,groups); toc;
		
        filenameSave = [folder '/' stem num2str(chain) '_run_' num2str(savenum) '.mat'];
        
        
        if thin_each>1 % Thin files
            thin = 1:thin_each:1000;
            all_B_vec=all_B_vec(:,thin);
            all_Sigma=all_Sigma(:,:,thin);
            all_N = all_N(:,thin);
            all_Ng = all_Ng(:,thin);
            all_V = all_V(:,:,thin);
            
            %all_V.s=all_V.s(:,:,thin);
            %all_V.n=all_V.n(:,:,thin);
            %all_V.naive=all_V.naive(:,thin);
            fprintf('%s\n',filenameSave);
        end
        
        save(filenameSave, 'all*','*_vals');
        fprintf('%s Saved\n',filenameSave);
           
    end
end
run_next_list = run_this_list + numel(run_this_list);

end

