
clearvars -except tasks do_specifications;

%% Load the file(s)

lottery_data = importdata('../lottery_file.csv', ',');
%% Assign variables;
for columnNumber = 1:numel(lottery_data.colheaders)
    exec=strcat(lottery_data.colheaders(columnNumber),'=lottery_data.data(:,columnNumber);');
    eval(exec{1});
end

clear('exec');
ns = numel(student_id);

%% Student-Home Language;

lang_dict = {{[100 267],'English'},...
    {7, 'Spanish'},{6,'Portuguese'},{[225 410],'Creole'}};

slang(isnan(slang))=999;
slang_aux = sparse(1:ns,slang,ones(ns,1));

hlang(isnan(hlang))=999;
hlang_aux = sparse(1:ns,hlang,ones(ns,1));

slan.name ={}; hlan.name ={};
slan.data =[]; hlan.data =[];
for lnum = 1:numel(lang_dict);
    slan.name(lnum) = lang_dict{lnum}(2);
    slan.data(:,lnum) = sum(slang_aux(:,lang_dict{lnum}{1}),2);
    hlan.name(lnum) = lang_dict{lnum}(2);
    hlan.data(:,lnum) = sum(hlang_aux(:,lang_dict{lnum}{1}),2);
end
slan.name{lnum+1} = 'Others';
slan.data(:,lnum+1) = 1-sum(slan.data,2);
hlan.name{lnum+1} = 'Others';
hlan.data(:,lnum+1) = 1-sum(hlan.data,2);


%% Ethnicity

eth_dict = {{2,'Black'},{3, 'Asian'},{5,'Hispanic'},{6,'White'}};

eth(isnan(eth))=999;
eth_aux = sparse(1:ns,eth,ones(ns,1));

ethn.colheaders ={}; 
ethn.data =[];
for enum = 1:numel(lang_dict);
    ethn.name(enum) = eth_dict{enum}(2);
    ethn.data(:,enum) = sum(eth_aux(:,eth_dict{enum}{1}),2);
end
ethn.name{lnum+1} = 'Others';
ethn.data(:,lnum+1) = 1-sum(ethn.data,2);

%% School - Student Characteristics;
school_dict = {{10000,'Baldwin','distance10'},...
    {13000,'Haggerty','distance13'},...
    {15000,'Amigos','distance15'},...
    {15100,'Amigos Bilingual','distance15'},...
    {17000,'MLK','distance17'},...
    {18000,'King Open','distance18'},...
    {20000,'Morse','distance20'},...
    {21000,'Peabody','distance21'},...
    {23000,'Tobin','distance23'},...
    {23040,'Tobin K4','distance23'},...
    {23050,'Tobin K5','distance23'},...
    {24000,'Flet Mayn','distance24'},...
    {25000,'Graham Parks','distance25'},...
    {27000,'Kenn Long','distance27'},...
    {28000,'Cambridgeport','distance28'},...
    {29000,'King Open Ola','distance29'}};

sch1 = sch_prog1 - paidlunch; % Just the school;
sch2 = sch_prog2 - paidlunch;
sch3 = sch_prog3 - paidlunch;
assert(all(mod([sch1;sch2;sch3],10)==0 | isnan([sch1;sch2;sch3]))); %NaN or multiples of 10.

% Distances
dist_string = 'dists =[';
for sch = 1:numel(school_dict)
    dist_string = [dist_string ' ' school_dict{sch}{3}];
end
eval([dist_string '];']);

% Proximity
prox_aux=sparse([find(prox1);find(prox2);find(prox3)],...
    [sch1(prox1==1);sch2(prox2==1);sch3(prox3==1)],...
    ones(sum(prox1+prox2+prox3),1),ns,29000);
prox.code = [];
prox.name = {};
for sch = 1:numel(school_dict)
    prox.code(sch) = school_dict{sch}{1};
    prox.name{sch} = school_dict{sch}{2};
end
prox.data = prox_aux(:,prox.code);

% Ranked Schools;
rank_aux = sparse(find(~isnan(sch1)),sch1(~isnan(sch1)),ones(sum(~isnan(sch1)),1),ns,29000);
rank_aux = rank_aux + sparse(find(~isnan(sch2)),sch2(~isnan(sch2)),2*ones(sum(~isnan(sch2)),1),ns,29000);
rank_aux = rank_aux + sparse(find(~isnan(sch3)),sch3(~isnan(sch3)),3*ones(sum(~isnan(sch3)),1),ns,29000);

rank = prox;
rank.data = rank_aux(:,rank.code);

% Sibling 
sibl_aux=sparse([find(sib1);find(sib2);find(sib3)],...
    [sch1(sib1==1);sch2(sib2==1);sch3(sib3==1)],...
    ones(sum(sib1+sib2+sib3),1),ns,29000);
sibl = prox;
sibl.data = sibl_aux(:,sibl.code);


%% Actual Assignment
asg_sch_prog = asg_sch_prog - paidlunch;
asgn_aux = sparse(find(~isnan(asg_sch_prog)),asg_sch_prog(~isnan(asg_sch_prog)),ones(sum(~isnan(asg_sch_prog)),1),ns,29000);
asgn = prox;
asgn.data = asgn_aux(:,asgn.code);

%% Actual Enrollment
reg_sch(isnan(reg_sch))=99; % What's a NaN?
regs_aux = sparse(1:ns,reg_sch*1000,ones(ns,1),ns,99000);
regs = prox;
regs.data = regs_aux(:,regs.code);


% Decision: Do proximity and sibling priority extend to bilingual programs?

%% Inelegibility

% Amigos Bilingual: If you didn't rank the bilingual program you were not elegile
ineg_ab = rank_aux(:,15100)==0;
ineg_aux =  sparse(find(ineg_ab),15100*ones(sum(ineg_ab),1),ones(sum(ineg_ab),1),ns,29000);
% Tobin. In years 2007 and 2008 your age determined whether you were
% eligible for K4 or K5.
ineg_t4 = year<2007 | elig_age >4;
ineg_t5 = year<2007 | elig_age <5;
ineg_t0 = year>=2007;
ineg_aux = ineg_aux + sparse(find(ineg_t4),23040*ones(sum(ineg_t4),1),ones(sum(ineg_t4),1),ns,29000);
ineg_aux = ineg_aux + sparse(find(ineg_t5),23050*ones(sum(ineg_t5),1),ones(sum(ineg_t5),1),ns,29000);
ineg_aux = ineg_aux + sparse(find(ineg_t0),23000*ones(sum(ineg_t0),1),ones(sum(ineg_t0),1),ns,29000);
ineg = prox;
ineg.data = ineg_aux(:,ineg.code);

%assert(all(ineg.data(:).*rank.data(:)==0));
%ineg.data.*rank.data


%% Generate prxd: Our proximities according to distances:
select_prxd = 'newer' % 'original','new','newer'; 
    %original = it was wrong. Sorted schools by distance and prioritized
    %           top 2 irrespective of whether 2 and 3 had the same distance
    %new = if Amigos and MLK are 1 and 2, student gets priority in those and nothing else
    %newer = if Amigos and MLK are 1 and 2, student gets priority in those two and another close school

build_prxd;

%% Capacities;
capacity = importdata('../capacity.csv', ',');
[~,vn]=ismember( {'year'  'tot_capacity'  'paidlunch'  'capacity'  'sch_prog'},capacity.colheaders);
yr = capacity.data(:,vn(1));
tc = capacity.data(:,vn(2));
pl = capacity.data(:,vn(3));
pc = capacity.data(:,vn(4));
sp = capacity.data(:,vn(5));

prog_cap=sparse(yr,sp,pc);
scho_cap=sparse(yr,sp,tc);

paid = full(prog_cap(min(year):max(year),prox.code+1));
free = full(prog_cap(min(year):max(year),prox.code));
tota = full(scho_cap(min(year):max(year),prox.code));
totb = full(scho_cap(min(year):max(year),prox.code+1));
assert(isequal(tota,totb)); % Verify that the two entries for total capacities coincide;
kpct.data = permute(cat(3,paid,free,tota),[2 3 1]);
kpct.name = prox.name;
kpct.code = prox.code;
kpct.cols = {'Paid lunch capacity','Free lunch capacity','Total school capacity'};
kpct.year = min(year):max(year);


%% Save

save('data.mat','prox','rank','ethn','slan','hlan','ineg','sibl','dists',...
    'paidlunch','student_id','year','stud','prxd','prxd','kpct','asgn','regs',...
    'med_fam_inc','med_hhold_inc')




