function make_table_mcmc(folder,file,print_map_names,varUnits,extrarows)

load([folder '/master']);
load([folder '/' file]);

if ~(bootstrapSE_flag || (~iscell(method) && strcmp(method,'truthful')))
    fprintf('Missing Bootstrap SE!\n');
    keyboard;
end

if nargin<2
	print_map = 1:n_progs;
else
	[~,print_map] = ismember(print_map_names,school_names);
end

	


%% Make the regression table
% Columns
% 1  : Covariate name
% 2 to 2*n_progs+1  : Mean and std of coefficients
fid = fopen([folder '/coefficients.csv'], 'w');

n_progs = length(school_names);
for i = 1:n_progs
    fprintf(fid, ', %s, ', school_names{print_map(i)});
end
fprintf(fid, '\n');

n_covsi = length(covariates);
n_coeff=numel(mean_B);
extra = mod(n_coeff,n_covsi);

if nargin<4 || isempty(varUnits), varUnits=ones(n_covsi,1); end
if nargin<5
	for coeff=1:extra
		extrarows{coeff}=['Extra ' num2str(coeff)]; 
	end
end
 

B = reshape(mean_B(1:(end-extra)), n_covsi, []);
e = sqrt(reshape(mean_error2_B(1:(end-extra)), n_covsi, []));

for coeff = 1:n_covsi
    fprintf(fid, '%s', covariates{coeff});
    for i = 1:n_progs
    	fprintf(fid, ', %4.2f, [%4.2f]', B(coeff, print_map(i))*varUnits(coeff), e(coeff, print_map(i))*varUnits(coeff));
    end
    fprintf(fid, '\n');
end

for coeff = 1:extra
	fprintf(fid, '%s, %4.2f, [%4.2f] \n', extrarows{coeff} , mean_B(n_covsi*n_progs+coeff), sqrt(mean_error2_B(n_covsi*n_progs+coeff)));
end


%fprintf(fid, 'Distance');
%for i = 1:n_progs
%    fprintf(fid, ', -1, ');
%end
%fprintf(fid, '\n\n');

fprintf(fid, 'Unobservable s.d.');
for i = 1:n_progs
    delta_method = 0.5 * sqrt(mean_error2_S(print_map(i), print_map(i))/mean_S(print_map(i), print_map(i)));
    fprintf(fid, ', %4.2f, [%4.2f]', sqrt(mean_S(print_map(i), print_map(i))), delta_method );
end
fprintf(fid, '\n\n');

% Now write the notes
fprintf(fid, 'N = %d\n', n_students);
if ~isempty(excluded_ethnicity)
   fprintf(fid, 'Excluded ethnicity is %s.  ', excluded_ethnicity);
end
if ~isempty(excluded_language)
   fprintf(fid, 'Excluded language is %s.  ', excluded_language);
end
fprintf(fid, '\n');

if ~iscell(method)
    fprintf(fid, 'The constraints are created using ''%s''.\n', method);
end

fclose(fid);

%% Make Agreggate Mean Utility 
fid = fopen([folder '/otherStuff.csv'], 'w');

n_progs = length(school_names);
for i = 1:n_progs
    fprintf(fid, ', %s, ', school_names{print_map(i)});
end
fprintf(fid, ',Outside Option,\n');
n_schools = numel(school_names);

B = reshape(mean_stuff, n_schools+1, []); B=B';
e = sqrt(reshape(mean_error2_stuff, n_schools+1, [])); e=e';

for coeff = 1:3
    fprintf(fid, '%s', linearf.name{coeff});
    for i = 1:n_progs
    	fprintf(fid, ', %4.2f, [%4.2f]', B(coeff, print_map(i)), e(coeff, print_map(i)));
    end
    fprintf(fid, ', %4.2f, [%4.2f]\n',B(coeff, end), e(coeff, end));
end

% Now write the notes
fprintf(fid, 'N = %d\n', n_students);
if ~isempty(excluded_ethnicity)
   fprintf(fid, 'Excluded ethnicity is %s.  ', excluded_ethnicity);
end
if ~isempty(excluded_language)
   fprintf(fid, 'Excluded language is %s.  ', excluded_language);
end
fprintf(fid, '\n');
if ~iscell(method)
    fprintf(fid, 'The constraints are created using ''%s''.\n', method);
end

fclose(fid);


%% Now make the correlation file
fid = fopen([folder '/corrMat.csv'], 'w');
for i = 1:n_progs
    fprintf(fid, ', %s', school_names{print_map(i)});
end
fprintf(fid, '\n');

for i = 1:n_progs
    fprintf(fid, '%s', school_names{print_map(i)});
    for j = 1:n_progs
    	if j > i
			fprintf(fid, ', ');
	    else
			fprintf(fid, ', %6.3f', mean_S(print_map(i), print_map(j)));
			delta_method = 0.5 * sqrt(mean_error2_S(print_map(i), print_map(i))/mean_S(print_map(i), print_map(i)));
			t = mean_S(print_map(i), print_map(j))/sqrt(mean_error2_S(print_map(i), print_map(j)));
			if t>1.644, fprintf(fid, '*');
				if t>1.96, fprintf(fid, '*');
					if 	t>2.575, fprintf(fid, '*');
					end
				end
			end
			
		end
    end
    fprintf(fid, '\n');
end

fclose(fid);

