% Set deault values if not provided.
if ~exist('do_burn_in','var') || isempty(do_burn_in), do_burn_in=5000; end
if ~exist('do_NITER','var') || isempty(do_NITER), do_NITER=40000; end
if ~exist('do_stem','var') || isempty(do_stem), do_stem='score_'; end

is_mixture = strcmp(do_spec,'mixture');
% Load master file and transform Xcell in X if needed.
X=[];
if is_mixture
    load([folder '/master.mat'],'X','distances','n_progs','esample','Xcell','consolidate','constraint_matrix');
    orig.constraint_matrix=constraint_matrix;
    estm_lott = load(method,'lott','tag');
    load('../estimation/data','rank','paidlunch');
else
    load([folder '/master.mat'],'X','distances','n_progs','esample','Xcell','consolidate','tag');
    estm_lott = load(method,'lott'); estm_lott.tag = tag;
    load('../estimation/data','rank');
end
elot = estm_lott.lott;
% Construct X, if necessary.
if isempty(X), X = blkdiag(Xcell{:}); end
n_coeff = size(X, 2);

% Small number for finding a V in the cone
my_eps =1e-7;

% Drop students that are not included in the sample;
new_esample = esample(smpl);
idx_bs=smpl(new_esample);

% Init Values
init_vals = load([folder '/chain_summ_X.mat' ],'mean_B','mean_S');
S= init_vals.mean_S;

% Constructing constraint_matrix and init_vals.V;
constraint_matrix=cell(numel(idx_bs),1+is_mixture);
if is_mixture, N=zeros(numel(idx_bs),1); end

rationalized_students=true(numel(idx_bs),1);
V=zeros(numel(idx_bs),n_progs);
fix_mu = X * init_vals.mean_B;
fix_mu = reshape(fix_mu, size(distances,1), n_progs) - distances;
A=sparse(1:numel(consolidate.map),consolidate.map,ones(numel(consolidate.map),1));

run_grb = ~exist('pu','var');

% Start computing the constraint matrix;
for s=unique(idx_bs)
    if mod(s,100)==0
       disp(['Student number ' num2str(s) ' of ' num2str(size(rank.data,1))]); 
    end
    % Reshape the ranks
    [~,ranks]=ismember([1,2,3]',rank.data(s,:));    
    % Figure out location of chosen lottery
    if ismember(method,{'current_year','current_year_av','last_year','current_year_ts','current_year_av_ts','last_year_ts','current_year_ts_wl','current_year_ts3'})
        stu_loc=student_type(s);
    elseif isequal(method,'fullinfo')
        stu_loc=s;
    else
        disp('Unrecognized method: type dbcont to continue using standard routine   ');
        stu_loc=student_type(s);
    end
    
    
    % Need to replace NaN to 0
    choice_set=lott(stu_loc).choice; 
    choice_set(isnan(choice_set))=0; elot(stu_loc).choice(isnan(elot(stu_loc).choice))=0;
    assert(all(choice_set(:)==elot(stu_loc).choice(:)));
    if ~isempty(choice_set)
        [check,chosen_lott]=ismember(ranks',choice_set,'rows');

        if ~check
           disp(['Whoops! check out student ' num2str(s)]);
           pause;
        end
        [d1,d2] = size(choice_set);
        wishful=sparse(repmat(1:d1,1,d2),choice_set(:)+1,repmat(d2:-1:1,d1,1));
        wishful=wishful(:,2:end); wishful = wishful/6;
        wishful(isnan(wishful))=0;
        % Compute the difference
        chosen_p=((1-shr-shx)*lott(stu_loc).p(chosen_lott,:) + shr*elot(stu_loc).p(chosen_lott,:) + shx*wishful(chosen_lott,:)); 
        all_lott=(1-shr-shx)*lott(stu_loc).p + shr*elot(stu_loc).p + shx*wishful;
        con_mat=repmat(chosen_p,d1,1)-all_lott;
        
        % Use consolidate and drop redundant rows;
        ranked_schools = rank.data(s,:)>0;
        keep_cols=sum(A(:,find(ranked_schools*A)),2)==0 | ranked_schools';
        con_mat_orig=con_mat(:,keep_cols)*A(keep_cols,:);
        % Clean rows
        con_mat = con_mat_orig;
        con_mat(estm_lott.tag{s},:)=[];
        con_mat = unique(con_mat,'rows');
        absrow = sum(abs(con_mat),2);
        con_mat(absrow<1e-8,:) = [];
        con_mat = unique(con_mat,'rows');
        if is_mixture, this_N = 1; end
        con_mat = sparse(con_mat);
        
        if all(con_mat*fix_mu(s,:)'>=0)
            result.x=fix_mu(s,:)';
        elseif run_grb

            model=struct();
            model.A=con_mat;
            model.sense='>';
            model.lb=repmat(-Inf,size(model.A,2),1);   % Gurobi defaults to a lower bound of 0 - need to allow for outside option to be worse
            model.rhs=my_eps*ones(size(model.A,1),1);
            model.obj=-2*fix_mu(s,:)/S;
            model.Q = sparse(inv(S));
            
            params = struct();
            params.Presolve = 0;
            params.outputflag = 0;
            params.FeasibilityTol = 1e-9;

            try
                result = gurobi(model, params);
            catch
                result.status = 'INFEASIBLE';
            end
            if ~strcmp(result.status, 'OPTIMAL') || result.objval > sum(sum(model.Q))*10^2 || ~(all(con_mat*result.x>0));
                if is_mixture,
                    model.A = sparse(orig.constraint_matrix{s,2});
                    model.lb=repmat(-Inf,size(model.A,2),1);   % Gurobi defaults to a lower bound of 0 - need to allow for outside option to be worse
                    model.rhs=my_eps*ones(size(model.A,1),1);
                    result = gurobi(model, params);
                    this_N = 2;
                    if ~strcmp(result.status, 'OPTIMAL')==1, keyboard; end
                 else
                    fprintf('[%d : %s : %d]',s,result.status,result.objval)
                    rks = consolidate.map(ranks(ranks>0));
                         
                    v=min(fix_mu(s,:),-1e-5); v(rks)=0; 
                    if numel(rks)>2,v(rks(3))=1e-15; end
                    aux=(con_mat*v');
                    if numel(rks)>1
                        c=con_mat(:,rks(2));
                        if any(c>0),
                            v(rks(2))=-min(aux(c>0)./c(c>0))+1e-15;
                        end
                    end
                    aux=(con_mat*v'); c=con_mat(:,rks(1));
                    ix = true(size(result.x)); ix(rks(1))=false;
                    v1 = 1/2*(model.obj(~ix)+v(ix)*model.Q(ix,~ix))/model.Q(~ix,~ix);
                    v(rks(1))=max([-min(aux(c>0)./c(c>0))+1e-15,v1]);
                    %objval = (v-fix_mu(s,:))*model.Q*(v-fix_mu(s,:))' - fix_mu(s,:)*model.Q*fix_mu(s,:)';
                    objval = model.obj*v'+v*model.Q*v';
                    if objval< result.objval || (all(con_mat*v'>0) && ~(all(con_mat*result.x>0)))
                        result.x = v'; result.objval = objval;
                        fprintf('[%d: %e : %d]',s, min(con_mat*result.x),result.objval)
                    end
                    if ~(all(con_mat*result.x>0)),
                        fprintf('[Failed %d : %e]',s,min(con_mat*result.x))
                    end
                     fprintf('\n')
                 end 
            end
        else
            result.x = zeros(size(con_mat,2),1);
            if is_mixture, this_N = 2; end
        end
        
        for ss = find(idx_bs==s)
            V(ss,:)=result.x(:);
            constraint_matrix{ss,1}=con_mat;
            if is_mixture,
                constraint_matrix{ss,2}=sparse(orig.constraint_matrix{s,2});
                N(ss) = this_N;
            end
        end
    end
end
init_vals.V=V;
if is_mixture, init_vals.N=N; end

if ~run_grb   % Assign initial conditions for last runs.
    init_vals.V = pu.final_vals.V;
    if is_mixture, init_vals.N = pu.final_vals.N; end 
end
% Done computing the constraint matrix;
    
Xsmpl=repmat(idx_bs(:),1,n_progs)+numel(esample)*ones(sum(new_esample),1)*(0:n_progs-1);
X=X(Xsmpl(:),:);
distances = distances(idx_bs,:);
if is_mixture, groups = paidlunch(idx_bs)+1; end


% Diagnose Errors
assert(sum(sum(isnan(X))) + sum(sum(isinf(X))) == 0); %infs or nans in X
assert(sum(sum(isnan(distances))) + sum(sum(isinf(distances))) == 0); %infs or nans in distances
% May fail; assert(sprank(X)==min(size(X))); % Full rank design matrix;

fprintf('Ready to run chain. do_NITER: %d!\n',do_NITER);
burn_in = do_burn_in;
per_run = do_per_run; runs = floor(do_NITER/per_run); runs_burn = ceil(burn_in/per_run); runs_keep =runs-runs_burn;
if is_mixture,
    [ ~, priors ] = gen_chain_mcmc_mex_mix(folder);
    mean_all_Ng=0;
else
    [ ~, priors ] = gen_chain_mcmc_mex(folder); 
end 
final_vals = init_vals; mean_B_vec=0;mean_Sigma=0;
for r =1:runs
    if is_mixture,
        tic;[all_B_vec, all_Sigma, ~, ~, all_Ng, final_vals, init_vals] = gibbsSamplerMixMex(X, constraint_matrix, priors, [], distances, per_run, final_vals,groups); toc;
    else
        tic;[all_B_vec, all_Sigma, ~, final_vals, init_vals] = gibbs_sampler_mex(X, constraint_matrix, priors, [], distances, per_run, final_vals); toc;
    end
    if r>runs_burn
        mean_B_vec=mean_B_vec+mean(all_B_vec,2)/runs_keep;
        mean_Sigma=mean_Sigma+mean(all_Sigma,3)/runs_keep;
       if is_mixture, mean_all_Ng=mean_all_Ng+mean(all_Ng,2)/runs_keep; end
    end
end

chain = do_draw_num;
final_vals.version = 1*all(~isnan(final_vals.V(:)));
filenameSave = [folder '/' do_stem num2str(chain) '.mat'];
if is_mixture
    save(filenameSave, 'mean_B_vec','mean_Sigma','mean_all_Ng','*_vals','do_NITER','burn_in','shr','shx');
else
    save(filenameSave, 'mean_B_vec','mean_Sigma','*_vals','do_NITER','burn_in','shr','shx');
end
fprintf('%s Saved\n',filenameSave);
        
        
    
