



for spec = do_specifications
    
    clearvars -except do* tasks* spec
    
    % Tasks
    build_stuff = isfield(do,'build') && do.build;  % set to true for new specification
    run_gibbs   = isfield(do,'gibbs_chain') && numel(do.gibbs_chain)==1;
    if run_gibbs, chain=do.gibbs_chain; resumeAt = do.resumeAt;  end 
    read_chains = isfield(do,'read_chain') && do.read_chain; 
    show_diagnosis = isfield(do, 'show_diagnostics') && do.show_diagnostics;
    make_the_tables = isfield(do,'tables') && do.tables;
    build_rnd_chain = isfield(do,'build_rnd_chain') && do.build_rnd_chain;

    
    % Choose how you want to compute the A matrix and choose the language
    method = 'current_year_ts'; % How to form the A matrix: fullinfo, current_year, past_year, truthful
    language = 'hlan'; % hlan = language at home, slan = primary spoken language
    % Choose which ethnicities and languages to include (the first one is the no-dummy; everything else is "other")
    other_cov_list = {'Constant', 'Paid Lunch', 'Sibling'}; % Constant, Paid Lunch, Sibling
    eth_list = {'White', 'Black', 'Asian', 'Hispanic'}; % White, Black, Asian, Hispanic
    lang_list = {'English', 'Spanish', 'Portuguese'}; % English, Spanish, Portuguese, Creole

    % It requires the
    % variables language, other_cov_list, eth_list and lang_list, consolidate.
    consolidate.name = {'Baldwin','Haggerty',{'Amigos','Amigos Bilingual'}, 'MLK',...
        'King Open','Morse','Peabody',{'Tobin','Tobin K4','Tobin K5'},'Flet Mayn',...
        'Graham Parks', 'Kenn Long','Cambridgeport','King Open Ola'};

    switch spec{1}
        case 'truthful',        folder = 'truthful1';          method = 'truthful'; 
        case 'current_year',    folder = 'currentyear1_ts';    method = 'current_year_ts'; 
        case 'last_year',       folder = 'lastyear1_ts';       method = 'last_year_ts';
        case 'current_year_av', folder = 'currentyearav1_ts';  method = 'current_year_av_ts';
        case 'current_year_wl', folder = 'currentyear1_ts_wl'; method = 'current_year_ts_wl';
        case 'mixture',         folder = 'mixture1_ts';        methods = {'current_year_ts','truthful'};
                                % How to form the A matrix: fullinfo, current_year, past_year, truthful
                                constraint_matrix_from = {'currentyear1_ts','truthful1'};
    end
    

    %% Run it.
    switch spec{1}
        case 'mixture'  % Mixture code
            if build_stuff
                build_X; 
                if exist('constraint_matrix_from','var')
                    num_cm = numel(constraint_matrix_from);
                    cmt = []; 
                    Vcm=cell(num_cm,1);
                    scm=cell(num_cm,1);
                    for cm = 1:num_cm
                        getcm=load([constraint_matrix_from{cm} '/master'],'constraint_matrix','V','distances','stud');
                        cmt = [cmt getcm.constraint_matrix];
                        Vcm{cm} = getcm.V;
                        scm{cm} = getcm.stud;
                        fprintf('constraint matrix from: %s\n',constraint_matrix_from{cm})    
                        distances = getcm.distances;
                    end

                else
                    num_cm = numel(methods);
                    cmt = []; 
                    Vcm=cell(num_cm,1);
                    scm=cell(num_cm,1);
                    for cm = 1:num_cm
                        method = methods{cm}
                        eps =1e-2; build_constraints_eps; 
                        cmt = [cmt constraint_matrix];
                        Vcm{cm} = V;
                        scm{cm} = stud;
                    end
                end
                constraint_matrix = cmt;
                V = Vcm;
                
                esample = ~any(isnan(distances),2); % Students to include (drop the guy with NaN distance; and those who we could not rationalize)
                
                %for st = 1:numel(constraint_matrix.s)
                %    At = constraint_matrix.s{st}/constraint_matrix.n{st};
                %    constraint_matrix.u{st}= At(all(At>=0,2),:);
                %end
                X=[]; %uses default: X = blkdiag(Xcell{:});
                method = methods;
                gen_master_file; % This script just sees whether it's necessary to create a folder and saves 
            end
            %%
            thin_each = 20;

            % Create Groups;
            load('data.mat','paidlunch'); load([folder '/master.mat'],'esample'); groups = paidlunch(esample)+1;
            
            estimation_script_mixture;


        otherwise   % All other specs
            
            
            if build_stuff
                build_all;
                esample = ~any(isnan(distances),2);
                if ~strcmp(spec{1},'truthful'), esample & stud.rationalized; end % Students to include (drop the guy with NaN distance; and those who we could not rationalize)
                if strcmp(spec{1},'last_year'), esample = esample & year~=2004 & year~=2007; end 
                X=[]; %uses default: X = blkdiag(Xcell{:});
                gen_master_file; % This script just sees whether it's necessary to create a folder and saves 
            end
            %%
            thin_each = 20;

            
            estimation_script;
    end

end