#include "mex.h"
#include <math.h>
#include <string.h>
#include <sys/time.h>
#include <stdio.h>
#include <fenv.h>

/* #define MAXDUB mxGetInf() */
#define MAXDUB INFINITY

/* #define OMP_PARALLEL */
/* #define TIME_LOOP */

double sign (double);
double erfcinv(double);

#ifdef TIME_LOOP
void toc(const struct timeval, const char*);
struct timeval tic();
#endif

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  /* feenableexcept(FE_INVALID | FE_OVERFLOW); */
  /* feenableexcept(FE_DIVBYZERO); */
  /* Following checks commented for speed */
  /* if (nrhs != 7) { */
  /*   mexErrMsgTxt("Number of inputs should be 7\n"); */
  /* } */
  /* if (nlhs != 1) { */
  /*   mexErrMsgTxt("Number of inputs should be 1\n"); */
  /* } */
  /* Implement more checks here if required */

  /* retrive vOld */
  double *vOld = mxGetPr(prhs[0]);
  int nStudents = mxGetM(prhs[0]);
  int nProgs = mxGetN(prhs[0]);

  /* retrive allMu */
  double *allMu = mxGetPr(prhs[2]);
  /* retrive distances */
  double *distances = mxGetPr(prhs[3]);
  /* retrive drawsMat */
  double *drawsMat = mxGetPr(prhs[4]);
  /* retrive condSdVec */
  double *condSdVec = mxGetPr(prhs[6]);

  /* create output matrix */
  plhs[0] = mxCreateDoubleMatrix(nStudents, nProgs, mxREAL);
  double *v = mxGetPr(plhs[0]);

  /* Main loop */
#ifdef TIME_LOOP
  struct timeval totalLoop = tic();
#endif

#ifdef OMP_PARALLEL
  int nCPU = omp_get_num_procs();
  /* int nCPU = 2; */
  omp_set_num_threads(nCPU);
#pragma omp parallel for
#endif
  for (int i = 0; i < nStudents; ++i) {
    /* Getting pointers to arrays of sparse matrix */
    mxArray *constraintMatrix = mxGetCell(prhs[1], i);
    double *cMatrix = mxGetPr(constraintMatrix);
    size_t *jc = mxGetJc(constraintMatrix);
    size_t *ir = mxGetIr(constraintMatrix);
    const int nCol = mxGetN(constraintMatrix);
    const int nRow = mxGetM(constraintMatrix);

    double *mu = malloc(nProgs*sizeof(double));
    double *drawsVec = malloc(nProgs*sizeof(double));
    double *vTilde = malloc(nProgs*sizeof(double));
    bool vTildeFlag = false;
    for (int j = 0; j < nProgs; ++j) {
      mu[j] = allMu[nStudents*j+i]-distances[nStudents*j+i]; /* cache miss? */
      drawsVec[j] = drawsMat[nStudents*j+i];
      vTilde[j] = vOld[nStudents*j+i];
      /* check for nan or inf in vTilde */
      if (!mxIsFinite(vTilde[j])) {
        vTildeFlag = true;
      }
    }
    if (vTildeFlag) {
      printf("Something weird is happening : NaN or Inf in v?.\n");
    }
    double *aSliced = malloc(nRow*sizeof(double));
    double *axVtilde = malloc(nRow*sizeof(double));
    double *bounds = malloc(nRow*sizeof(double));
    double *boundUpper = malloc(nRow*sizeof(double));
    double *boundLower = malloc(nRow*sizeof(double));
    /* loop to compute truncation points */
    for (int j = 0; j < nProgs; ++j) {
      memset(aSliced, 0.0, sizeof(double)*nRow); /*May depend on platform/compiler*/
      memset(axVtilde, 0.0, sizeof(double)*nRow);
      /* aSliced = A(:,j); axVtilde = A*vtilde */
      const int limit = jc[j+1];
      for (int k = jc[j]; k < limit; ++k) {
        aSliced[ir[k]] = cMatrix[k];
      }

      for(int l = 0; l < nCol; ++l) {
        const int last = jc[l+1];
        double vVlaue = vTilde[l];
        for (int k = jc[l]; k < last; ++k) {
          axVtilde[ir[k]] += cMatrix[k]*vVlaue;
        }
      } 
      for (int k =0; k < nRow; ++k) {
        bounds[k] = -1.0*(axVtilde[k]-aSliced[k]*vTilde[j])/aSliced[k];
      }
      bool upperFlag = false;
      bool lowerFlag = false;
      for (int k = 0; k < nRow; k++) {
        if (aSliced[k] < 0.0) {
          upperFlag = true;
          break;
        }
      }
      double upper, lower;
      bool upperInfFlag = false, lowerInfFlag = false;
      if (!upperFlag) {
        upper = MAXDUB;
        upperInfFlag = true;
      } else {
        int upperBoundCount = 0;
        for (int k = 0; k < nRow; ++k) {
          if (aSliced[k] < 0.0) {
            boundUpper[upperBoundCount++] = bounds[k];
          }
        }
        upper = boundUpper[0];
        for (int k = 1; k < upperBoundCount; ++k) {
          if (boundUpper[k] < upper) {
            upper = boundUpper[k];
          }
        }
      }

      for (int k = 0; k < nRow; k++) {
        if (aSliced[k] > 0.0) {
          lowerFlag = true;
          break;
        }
      }
      if (!lowerFlag) {
        lower = -1.0*MAXDUB;
        lowerInfFlag = true;
      } else {
        int lowerBoundCount = 0;
        for (int k = 0; k < nRow; ++k) {
          if (aSliced[k] > 0.0) {
            boundLower[lowerBoundCount++] = bounds[k];
          }
        }
        lower = boundLower[0];
        for (int k = 1; k < lowerBoundCount; ++k) {
          if (boundLower[k] > lower) {
            lower = boundLower[k];
          }
        }
      }
      double aSliceSum = 0.0;
      for (int k = 0; k < nRow; ++k) {
        aSliceSum += aSliced[k];
      }
      if (lowerInfFlag && upperInfFlag && (fabs(aSliceSum) > 1e-6)) {
        printf("No bounds. \n");
      }
      /* get the conditional mean and variance */
      mxArray *localMatCell = mxGetCell(prhs[5], j);
      double *localMat = mxGetPr(localMatCell);
      double matCellvTildeMu = 0.0;
      for (int k = 0; k < nProgs; ++k) {
        if (k == j) {
          continue;
        } else {
          if (k < j) {
            matCellvTildeMu += localMat[k]*(vTilde[k]-mu[k]);
          } else {
            matCellvTildeMu += localMat[k-1]*(vTilde[k]-mu[k]);
          }
        }
      }
      double condMean = mu[j] + matCellvTildeMu; 
      double condSd = condSdVec[j];
      /* Draw from a truncated normal */
      double lowerZ = (lower - condMean)/condSd;
      double upperZ = (upper - condMean)/condSd;
      double lowerQ = 0.5*erfc(fabs(lowerZ)/M_SQRT2);
      double upperQ = 0.5*erfc(fabs(upperZ)/M_SQRT2);

      double rSmall = (-1.0*sign(sign(lowerZ)+0.5))*lowerQ+drawsVec[j]*\
                      (sign(sign(lowerZ)+0.5)*lowerQ-sign(sign(upperZ)+0.5)*upperQ);
      double rBig = ((lowerZ>0.0)?1.0:0.0)+drawsVec[j]*\
                    (((upperZ>0.0)?1.0:0.0)-((lowerZ>0.0)?1.0:0.0));
      double r;

      if ((rBig+rSmall) <= 0.5) {
        r = rBig+rSmall;
        vTilde[j] = -1.0*M_SQRT2*erfcinv(2.0*r)*condSd+condMean;
      } else {
        rBig = 1.0 - rBig;
        r = rBig-rSmall;
        vTilde[j] = M_SQRT2*erfcinv(2.0*r)*condSd+condMean;
      }

    }
    /* assign vTilde into v */
    for (int j = 0; j < nProgs; ++j) {
      v[nStudents*j+i] = vTilde[j];
    }
    /* Ensure consistency of draws */
    memset(axVtilde, 0.0, sizeof(double)*nRow);
    for(int l = 0; l < nCol; ++l) {
      int last = jc[l+1];
      for (int k = jc[l]; k < last; ++k) {
        axVtilde[ir[k]] += cMatrix[k]*vTilde[l];
      }
    } 
    for (int k = 0; k < nRow; ++k) {
      if (axVtilde[k] < 0.0) {
        printf("weird! I ended up with a vector outside the cone. \n");
      }
    }
    /* Free allocated memory */
    free(mu);
    free(drawsVec);
    free(vTilde);
    free(aSliced);
    free(axVtilde);
    free(bounds);
    free(boundUpper);
    free(boundLower);
  }
#ifdef TIME_LOOP
  toc(totalLoop, "Time taken by the total loop ");
#endif
  return;
}

double sign (double x) {
  if (x > 0.0) {
    return(1.0);
  } else {
    if (x == 0) {
      return(0.0);
    } else {
      return(-1.0);
    }
  }
}

#ifdef TIME_LOOP
void toc(const struct timeval t1, const char *text) {
  struct timeval t2;
  gettimeofday(&t2, NULL);
  double elapsedTime = (double)(t2.tv_sec - t1.tv_sec);
  elapsedTime += (double)((t2.tv_usec - t1.tv_usec)/(1e6));
  printf("%s : %f (s)\n", text, elapsedTime);
}

struct timeval tic() {
  struct timeval t1;
  gettimeofday(&t1, NULL);
  return t1;
}
#endif

double erfcinv(double p) {
  double x, err, t, pp;

  if (p >= 2.) return -INFINITY;
  if (p <= 0.0) return INFINITY;

  pp=(p < 1.0)? p:2.-p;
  t=sqrt(-2.*log(pp/2));

  x= -0.70711*((2.30753+t*0.27061)/(1+t*(0.99229+t*0.04481))-t);

  for (int j=0;j<2;j++) {
      err=erfc(x)-pp;
      x += err/(1.12837916709551257*exp(-x*x)-x*err);
    }
  return (p<1.0 ? x:-x);
}
