#include "mex.h"
#include <math.h>
#include <string.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <fenv.h>

#include <omp.h>

#define MAXDUB INFINITY

double sign (double);
double erfcinv(double);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  /* retrive vOld */
  double *vOld = mxGetPr(prhs[0]);
  const unsigned int nStudents = mxGetM(prhs[0]);
  const unsigned int nProgs = mxGetN(prhs[0]);
  /* const unsigned int nCones = mxGetN(prhs[1]); */
  const unsigned int nCones = 2;
  
  /* retrive allMu */
  double *allMu = mxGetPr(prhs[2]);
  /* retrive distances */
  double *distances = mxGetPr(prhs[3]);
  /* retrive drawsMat */
  double *drawsMat = mxGetPr(prhs[4]);
  /* retrive condSdVec */
  double *condSdVec = mxGetPr(prhs[6]);
  /* retrive pc */
  double *pc = mxGetPr(prhs[7]);

  /* create output matrix */
  plhs[0] = mxCreateDoubleMatrix(nStudents, nProgs, mxREAL);
  double *v = mxGetPr(plhs[0]);
  plhs[1] = mxCreateDoubleMatrix(nStudents, 1, mxREAL);
  double *n = mxGetPr(plhs[1]);

  /* Main loop */
  int nCPU = omp_get_num_procs();
  /* printf("Number of CPU's : %d\n", nCPU); */
  /* int nCPU = 1; */
  omp_set_num_threads(nCPU);
  double *muGlobal = malloc(nCPU*nProgs*sizeof(double));
  double *drawsVecGlobal = malloc(nCPU*nProgs*sizeof(double));
  double *vTildeGlobal = malloc(nCPU*nProgs*sizeof(double));

#pragma omp parallel for
  for (unsigned int i = 0; i < nStudents; ++i) {
    int nT = omp_get_thread_num();
    double* mu = muGlobal+nT*nProgs;
    double* drawsVec = drawsVecGlobal+nT*nProgs;
    double* vTilde = vTildeGlobal+nT*nProgs;
    /* Create other required arrays */
    double pCone[2];
    pCone[0] = pc[i];
    pCone[1] = 1.0 - pCone[0];

    bool vTildeFlag = false;
    for (int j = 0; j < nProgs; ++j) {
      mu[j] = allMu[nStudents*j+i]-distances[nStudents*j+i]; 
      drawsVec[j] = drawsMat[nStudents*j+i];
      vTilde[j] = vOld[nStudents*j+i];
      /* check for nan or inf in vTilde */
      if (!mxIsFinite(vTilde[j])) {
        vTildeFlag = true;
      }
    }
    if (vTildeFlag) {
      printf("Something weird is happening : NaN or Inf in v?.\n");
      /* return; */
    }

    /* loop to compute truncation points */
    for (int j = 0; j < nProgs; ++j) {
      double upper[2];
      double lower[2];
      for (int m = 0; m < nCones; ++m) {
        /* Getting pointers to arrays of sparse matrix */
        mxArray *constraintMatrix = mxGetCell(prhs[1], i+m*nStudents);
        double *cMatrix = mxGetPr(constraintMatrix);
        size_t *jc = mxGetJc(constraintMatrix);
        size_t *ir = mxGetIr(constraintMatrix);
        const int nCol = mxGetN(constraintMatrix);
        const int nRow = mxGetM(constraintMatrix);

        double *aSliced = malloc(nRow*sizeof(double));
        double *axVtilde = malloc(nRow*sizeof(double));
        double *bounds = malloc(nRow*sizeof(double));
        double *boundUpper = malloc(nRow*sizeof(double));
        double *boundLower = malloc(nRow*sizeof(double));

        memset(aSliced, 0.0, sizeof(double)*nRow);
        memset(axVtilde, 0.0, sizeof(double)*nRow);
        /* aSliced = A(:,j); axVtilde = A*vtilde */
        const int limit = jc[j+1];
        for (int k = jc[j]; k < limit; ++k) {
          aSliced[ir[k]] = cMatrix[k];
        }
        for(int l = 0; l < nCol; ++l) {
          const int last = jc[l+1];
          double vValue = vTilde[l];
          for (int k = jc[l]; k < last; ++k) {
            axVtilde[ir[k]] += cMatrix[k]*vValue;
          }
        } 
        double tempValue = vTilde[j];
        for (int k =0; k < nRow; ++k) {
          bounds[k] = -((axVtilde[k]-aSliced[k]*tempValue)/aSliced[k]);
        }
        bool upperFlag = false;
        bool lowerFlag = false;
        for (int k = 0; k < nRow; k++) {
          if (aSliced[k] < 0.0) {
            upperFlag = true;
            break;
          }
        }
        bool upperInfFlag = false, lowerInfFlag = false;
        if (!upperFlag) {
          upper[m] = MAXDUB;
          upperInfFlag = true;
        } else {
          int upperBoundCount = 0;
          for (int k = 0; k < nRow; ++k) {
            if (aSliced[k] < 0.0) {
              boundUpper[upperBoundCount++] = bounds[k];
            }
          }
          upper[m] = boundUpper[0];
          for (int k = 1; k < upperBoundCount; ++k) {
            if (boundUpper[k] < upper[m]) {
              upper[m] = boundUpper[k];
            }
          }
        }

        for (int k = 0; k < nRow; k++) {
          if (aSliced[k] > 0.0) {
            lowerFlag = true;
            break;
          }
        }
        if (!lowerFlag) {
          lower[m] = -1.0*MAXDUB;
          lowerInfFlag = true;
        } else {
          int lowerBoundCount = 0;
          for (int k = 0; k < nRow; ++k) {
            if (aSliced[k] > 0.0) {
              boundLower[lowerBoundCount++] = bounds[k];
            }
          }
          lower[m] = boundLower[0];
          for (int k = 1; k < lowerBoundCount; ++k) {
            if (boundLower[k] > lower[m]) {
              lower[m] = boundLower[k];
            }
          }
        }
        if (upper[m] < lower[m]) {
          lower[m] = upper[m];
        } else {
          for (unsigned int k = 0; k < nRow; ++k) {
            if ((aSliced[k] == 0) && (bounds[k] > 0.0)) {
              lower[m] = upper[m];
              break;
            }
          }
        }
        double aSliceSum = 0.0;
        for (int k = 0; k < nRow; ++k) {
          aSliceSum += aSliced[k];
        }
        if (lowerInfFlag && upperInfFlag && (fabs(aSliceSum) > 1e-6)) {
          printf("No bounds. \n");
          /* return; */
        }

        free(aSliced);
        free(axVtilde);
        free(bounds);
        free(boundUpper);
        free(boundLower);
      }
      /* get the conditional mean and variance */
      mxArray *localMatCell = mxGetCell(prhs[5], j);
      double *localMat = mxGetPr(localMatCell);
      double matCellvTildeMu = 0.0;
      for (int k = 0; k < nProgs; ++k) {
        if (k == j) {
          continue;
        } else {
          if (k < j) {
            matCellvTildeMu += localMat[k]*(vTilde[k]-mu[k]);
          } else {
            matCellvTildeMu += localMat[k-1]*(vTilde[k]-mu[k]);
          }
        }
      }
      double condMean = mu[j] + matCellvTildeMu; 
      double condSd = condSdVec[j];
      /* Draw from a truncated normal */
      double upperZ[2], lowerZ[2];
      double upperQ[2], lowerQ[2];
      double length[2];

      double lengthSum = 0.0;
      for (unsigned int m = 0; m < nCones; ++m) {
        lowerZ[m] = (lower[m] - condMean)/condSd;
        upperZ[m] = (upper[m] - condMean)/condSd;
        lowerQ[m] = 0.5*erfc(fabs(lowerZ[m])/M_SQRT2);
        upperQ[m] = 0.5*erfc(fabs(upperZ[m])/M_SQRT2);
        length[m] = ((1.0*(upperZ[m]>0)- 1.0*(lowerZ[m]>0))-
            (sign(sign(upperZ[m])+0.5)*upperQ[m]-sign(sign(lowerZ[m])+0.5)*lowerQ[m]))*pCone[m];
        lengthSum += length[m];
      }
      for (unsigned int m = 0; m < nCones; ++m) {
        length[m] /= lengthSum;
      }
      double cumsum[2];
      cumsum[0] = length[0];
      cumsum[1] = cumsum[0]+length[1];
      int interval = -1;
      if (cumsum[0] > drawsVec[j]) {
        interval = 0;
      } else {
        if (cumsum[1] > drawsVec[j]) {
          interval = 1;
        }
      }
      if (interval < 0) {
        printf("Keyboard 1\n");
        /* return; */
      }
      cumsum[1] = cumsum[0]; 
      cumsum[0] = 0.0;
      double nd[2];
      nd[0] = drawsVec[j]/length[0];
      nd[1] = (drawsVec[j] - cumsum[1])/length[1];
      int interval2 = -1;
      if ((nd[0]>=0)&&nd[0]<=1) {
        interval2 = 0;
      }
      if ((nd[1]>=0)&&nd[1]<=1) {
        if (!interval2) {
          printf("Keyboard 2\n");
          /* return; */
        }
        interval2 = 1;
      }
      if (interval != interval2) {
        printf("Keyboard 3\n");
        /* return; */
      }
      double lowerZS = lowerZ[interval];
      double lowerQS = lowerQ[interval];
      double upperZS = upperZ[interval];
      double upperQS = upperQ[interval];
      double ndS = nd[interval];
      double rSmall = (-1.0*sign(sign(lowerZS)+0.5))*lowerQS+ndS*\
                      (sign(sign(lowerZS)+0.5)*lowerQS-sign(sign(upperZS)+0.5)*upperQS);
      double rBig = ((lowerZS>0.0)?1.0:0.0)+ndS*\
                    (((upperZS>0.0)?1.0:0.0)-((lowerZS>0.0)?1.0:0.0));
      double r;

      if ((rBig+rSmall) <= 0.5) {
        r = rBig+rSmall;
        vTilde[j] = -1.0*M_SQRT2*erfcinv(2.0*r)*condSd+condMean;
      } else {
        rBig = 1.0 - rBig;
        r = rBig-rSmall;
        vTilde[j] = M_SQRT2*erfcinv(2.0*r)*condSd+condMean;
      }

      bool inCone[2]; inCone[0] = true; inCone[1] = true;
      for (int m = 0; m < nCones; ++m) {
        /* Getting pointers to arrays of sparse matrix */
        mxArray *constraintMatrix = mxGetCell(prhs[1], i+m*nStudents);
        double *cMatrix = mxGetPr(constraintMatrix);
        size_t *jc = mxGetJc(constraintMatrix);
        size_t *ir = mxGetIr(constraintMatrix);
        const int nCol = mxGetN(constraintMatrix);
        const int nRow = mxGetM(constraintMatrix);

        double *axVtilde = malloc(nRow*sizeof(double));

        memset(axVtilde, 0.0, sizeof(double)*nRow);
        for(int l = 0; l < nCol; ++l) {
          const int last = jc[l+1];
          double vValue = vTilde[l];
          for (int k = jc[l]; k < last; ++k) {
            axVtilde[ir[k]] += cMatrix[k]*vValue;
          }
        } 
        for (int k = 0; k < nRow; ++k) {
          if (axVtilde[k] < 0.0) {
            inCone[m] = false;
            break;
          }
        }
        free(axVtilde);
      }
      if (!(inCone[0]||inCone[1])) {
        printf("weird! I ended up with a vector outside the cone. \n");
        /* return; */
      }
    }
    /* assign vTilde into v */
    for (int j = 0; j < nProgs; ++j) {
      v[nStudents*j+i] = vTilde[j];
    }
    /* Ensure consistency of draws */
    bool inCone[2];
    for (int m = 0; m < nCones; ++m) {
      /* Getting pointers to arrays of sparse matrix */
      mxArray *constraintMatrix = mxGetCell(prhs[1], i+m*nStudents);
      double *cMatrix = mxGetPr(constraintMatrix);
      size_t *jc = mxGetJc(constraintMatrix);
      size_t *ir = mxGetIr(constraintMatrix);
      const int nCol = mxGetN(constraintMatrix);
      const int nRow = mxGetM(constraintMatrix);

      double *axVtilde= malloc(nRow*sizeof(double));

      memset(axVtilde, 0.0, sizeof(double)*nRow);
      for(int l = 0; l < nCol; ++l) {
        const int last = jc[l+1];
        double vValue = vTilde[l];
        for (int k = jc[l]; k < last; ++k) {
          axVtilde[ir[k]] += cMatrix[k]*vValue;
        }
      } 
      bool coneFlag = true;
      for (int k = 0; k < nRow; ++k) {
        if (axVtilde[k] < 0.0) {
          coneFlag = false;
          break;
        }
      }
      inCone[m] = coneFlag;
      free(axVtilde);
    }
    double length[2];
    double cumlen[2];
    double randomValue = (double)rand()/(double)RAND_MAX;
    length[0] = (double)inCone[0]*pCone[0];
    length[1] = (double)inCone[1]*pCone[1];
    double sum = length[0]+length[1];
    length[0]/=sum;
    length[1]/=sum;
    cumlen[0] = length[0];
    cumlen[1] = length[0]+length[1]; //will be 1
    if (cumlen[0] > randomValue) {
      n[i] = 1;
    } else {
      n[i] = 2;
    }
  }
  free(muGlobal);
  free(drawsVecGlobal);
  free(vTildeGlobal);
}

double sign (double x) {
  if (x > 0.0) {
    return(1.0);
  } else {
    if (x == 0.0) {
      return(0.0);
    } else {
      return(-1.0);
    }
  }
}

double erfcinv(double p) {
  double x, err, t, pp;

  if (p >= 2.) return -INFINITY;
  if (p <= 0.0) return INFINITY;

  pp=(p < 1.0)? p:2.-p;
  t=sqrt(-2.*log(pp/2));

  x= -0.70711*((2.30753+t*0.27061)/(1+t*(0.99229+t*0.04481))-t);

  for (int j=0;j<2;j++) {
      err=erfc(x)-pp;
      x += err/(1.12837916709551257*exp(-x*x)-x*err);
    }
  return (p<1.0 ? x:-x);
}
