function V=sampleUtilsOriginal(V_old,constraint_matrix,all_mu,distances,draws_mat,mat_cell,cond_sd_vec)
% V_old  : n_students x n_progs (double) matrix
% constraint_matrix: cell array where c...{s} is the matrix A for student s
%                    so that Av > 0
% all_mu : n_students x n_progs (double) matrix
% distances : n_students x n_progs (double) matrix
% draws_mat : n_students x n_progs (double) matrix, each element is in [0,1]
keyboard;
% Initialize dimensions
n_students=size(V_old,1);
n_progs=size(V_old,2);

V=zeros(size(V_old));

for s = 1:n_students % This should be a parfor if you want to parallelize

        % Get constraint matrix for student s
        A = constraint_matrix{s};

        % Compute mu for student s
        mu = (all_mu(s, :) - distances(s, :))';

        % Get draws for student s
        draws_vec=draws_mat(s,:);

        % Init v_tilde
        vtilde = V_old(s, :)';
        if any(isnan(vtilde)) || any(isinf(vtilde))
            fprintf('Something weird is happening: NaN or Inf in v?.\n');
        end

        for i = 1:n_progs % length(vtilde)
          % Compute the truncation points            
          bounds = -(A * vtilde - A(:, i) * vtilde(i)) ./ A(:, i);            

          if all(A(:, i)>=0)
                upper = Inf;
            else
                upper = min(bounds(A(:, i)<0));
            end

            if all(A(:, i)<=0)
                lower = -Inf;
            else
                lower = max(bounds(A(:, i)>0));
            end
            if lower == -Inf  && upper == Inf && abs(sum(A(:, i))) > 1e-6 % So that we're not spitting this out whenever we have Amigos
                fprintf('No bounds.\n');
            end

            % Get the conditional mean and variance                        
            cond_mean = mu(i) + mat_cell{i}*(vtilde([1:i-1 i+1:n_progs]) - mu([1:i-1 i+1:n_progs]));
            cond_sd = cond_sd_vec(i);

            % Now draw from a truncated normal ~ N(cond_mean, cond_sd) on [lower, upper]
            lower_z = (lower - cond_mean) / cond_sd;
            upper_z = (upper - cond_mean) / cond_sd;

            % For numerical reasons, computing the CDFs and inverting them
            % from the left tail are better because MATLAB is does not
            % store 1-z precisely when z is very small
            lower_q = 0.5 * erfc(abs(lower_z)/ sqrt(2)); % normcdf(lower, cond_mean, cond_sd);
            upper_q = 0.5 * erfc(abs(upper_z)/ sqrt(2)); % normcdf(upper, cond_mean, cond_sd);

            r_small = (-sign(sign(lower_z)+0.5)* lower_q) + draws_vec(i) * (sign(sign(lower_z)+0.5)* lower_q -  sign(sign(upper_z)+0.5)* upper_q);
            r_big = 1*(lower_z>0) + draws_vec(i) * (1*(upper_z>0) - 1*(lower_z>0)); % The 1* ensure that Matlab transforms logical expressions into integers.

            if r_big+r_small<=0.5
              r = r_big+r_small;
              vtilde(i) = -sqrt(2) * erfcinv(2*r) * cond_sd + cond_mean; % norminv(r, cond_mean, cond_sd);
            else
                r_big = 1-r_big;
                r = r_big - r_small;
                vtilde(i) = sqrt(2) * erfcinv(2*r) * cond_sd + cond_mean; % norminv(r, cond_mean, cond_sd);
            end
        end

        V(s, :) = vtilde(:);

        % Ensure consistency of draws
        if any(A * vtilde < 0)
            fprintf('weird! I ended up with a vector outside the cone.\n'); 
        end
end
