% Master File
% Demand Analysis using Strategic Reports: An application to a school choice mechanism

%% The code for this project can be divided in the following tasks:
tasks.make_data                 = true; % Generates the main dataset. 
tasks.student_resample_draws    = true; 
tasks.lottery_estimation        = true;
tasks.prefs_estimation          = true;
tasks.run_bootstrap             = true;
tasks.da_counterfactual         = true; 
tasks.tables                    = {}; 

                %Paper:     {'Descriptive','Rejected','RD','Lotteries','MeanUtilities','Losses','Choice','DA'};
                %Appendix:  {'Lotteries','MeanUtilities'}

% The code splits by specifications: 
do_specifications = {'current_year'} %{'truthful','current_year_wl','mixture'} %{'truthful','current_year_wl','mixture'} % {'current_year_wl','last_year','mixture','current_year','current_year_av',};  
                %   {'truthful','current_year','last_year','current_year_av','current_year_wl','mixture'}

%% Preference estimation tasks:
tasks.pe.build                  = true; % set to true for new specification
tasks.pe.gibbs_chain            = 1;    % if empty gibbs sampler will not run
tasks.pe.resumeAt               = 0;     % resume at this file number
tasks.pe.read_chain             = true;  % reads chain results
tasks.pe.show_diagnostics       = true; % Show gibbs diagnostics 
tasks.pe.build_rnd_chain        = true; % 

%% Execution

% Preliminaries: 
tasks.dir = pwd; % Remember current working directory
for tt = 1:numel(do_specifications), if strcmp(do_specifications{tt},'mixture'), break; else, tt=0; end; end % finds mixture
if tt >0, 
    do_specs = [do_specifications(1:tt-1) {'mixture_naive','mixture_soph'} do_specifications(tt+1:end)];     % replace mixture by mixture_naive and mixture_soph
else 
    do_specs = do_specifications;   % no change if mixture is not included
end


% Perform tasks
% Format:   if this_task;  overwrite_work= 1/0; cd(directory)           script                      cd(tasks.dir)

     
if tasks.make_data,                             cd('estimation');       make_data;                  cd(tasks.dir); end
if tasks.student_resample_draws,                cd('est_lotteries');    gen_cutoffs_camb_ts;        cd(tasks.dir); end
if tasks.lottery_estimation, do_overwrite=1;    cd('est_lotteries');    gen_constraints_ts;         cd(tasks.dir); end
if tasks.prefs_estimation,   do =tasks.pe;      cd('estimation');       run_estimation;             cd(tasks.dir); end
if tasks.run_bootstrap,      do_overwrite=0;    cd('est_lotteries');    run_bootstrap_samples_ts;   cd(tasks.dir); end
if tasks.da_counterfactual,  do_overwrite=0;    cd('da_counter');       run_da_ts;                  cd(tasks.dir); end

for tt = 1:numel(tasks.tables)
    tabname = tasks.tables{tt};
    switch tabname
        % "Descriptive Statistics" Tables 2 and 3
        case 'Descriptive',                     cd('tables/DescriptiveStatistics');
                                                [status,cmdout] = system(['stata -b do run']),      cd(tasks.dir);
                                                
        % "School Popularity and Competitiveness" Table 4
        case 'Rejected',                        cd('tables/RejectedStudents'); 
                                                                        run_rejected;               cd(tasks.dir);
        
        % "Regression Discontinuity Estimates" Table 5
        case 'RD',                              cd('tables/RD');        run_rd;                     cd(tasks.dir);
        
        % "Assignment probabilities" Table 6 and D.1
        case 'Lotteries',                       cd('tables/EstimatedLotteries');
                                                                        run_table_lot;              cd(tasks.dir);
        
        % "Estimated Mean Utilities" Table 7 and 11 in the paper and Tables D.2, D.3 in the Appendix
        case 'MeanUtilities', do= struct('tables',1); cd('estimation'); run_estimation;             cd(tasks.dir);
        
        % "Losses from Truthful" Table 8
        case 'Losses',                          cd('tables/LossesIfTruthful'); 
                                                                        run_losses;                 cd(tasks.dir);
        
        % Table 9
        case 'Choice',                          cd('tables/ChoiceTable');
                                                                        run_tab_choice;             cd(tasks.dir);
        
        % "DA vs Cambridge" Tables 10 and 12                     
        case 'DA', run_prep_da_vs_camb=0;       cd('tables/DA');        run_tab_da;                 cd(tasks.dir);
    
    end
end

