
%clear;
%est_type = 'mixture2';
%override_method = true; method_id = 'truthful';

% Use current_year to compute actual assignment probabilities;
% asgn_lott = 'current_year';


% Load data
data=load('../../estimation/data');
est=load(['../../estimation/' est_type '/master']);
if override_method, est.method = method_id; end

tiebreak

% Names of the columns in the output table
tab_cons.name = {'Graham Parks','Haggerty','Baldwin','Morse','Amigos','Cambridgeport','King Open','Peabody','Tobin','Flet Mayn','Kenn Long','MLK','King Open Ola'};

%% Consolidate Utilities
assert(isequal(size(tab_cons.name),size(est.consolidate.name)));
[~,where_in_V]=unique(est.consolidate.map);
V_vec=V_vec(:,where_in_V,:);  % Modify V_vec

% Init
[n_stu,n_prog,n_sim]=size(V_vec); % Store the number of students

% Consolidate Chosen List
chosen_list_temp=zeros(size(chosen_list));
naiv_list_temp=zeros(size(naiv_list));
for pp=1:size(est.consolidate.map,1)
   chosen_list_temp(chosen_list==pp)=est.consolidate.map(pp);     
   naiv_list_temp(naiv_list==pp)=est.consolidate.map(pp);
end
chosen_list=chosen_list_temp;
naiv_list = naiv_list_temp;
clear chosen_list_temp naiv_list_temp;

% Consolidate Rank Data and Proximity Priorities
rank_data=zeros(n_stu,n_prog);
priority_data=zeros(n_stu,n_prog);
for pp=1:size(est.consolidate.map,1)
    assign_ranks=data.rank.data(:,pp);  % Get the ranks for the pp-th program    
    assert(all(rank_data(assign_ranks~=0,est.consolidate.map(pp))==0)); % Make sure that rank_data doesn't have this school prev ranked
    rank_data(assign_ranks~=0,est.consolidate.map(pp))=assign_ranks(assign_ranks~=0); % Assign ranks
    
    assign_priority=(data.prxd.data(:,pp)+data.sibl.data(:,pp))>0;  % Get the proximities for the pp-th program
    priority_data(:,est.consolidate.map(pp))=logical(assign_priority);
end

%% Create the Choice/Assignment table
clear('choice'); clear('naiv')
choice.true = zeros(n_stu,n_prog);
choice.report = zeros(n_stu,n_prog);
choice.report_data = zeros(n_stu,n_prog);
choice.choose_pp_priority = zeros(n_stu,n_prog);
choice.choose_other_priority = zeros(n_stu,n_prog);
choice.rank_and_kk = zeros(n_stu,n_prog);
choice.assign_p = zeros(n_stu,n_prog);
choice(2)=choice(1);
choice(3)=choice(1);
truthfulReport = zeros(n_stu,1);

naiv.assign=zeros(n_stu,n_prog);

% Iterate through students
for ss=1:n_stu
    this_V=[squeeze(V_vec(ss,:,:)); zeros(1,n_sim)];
    [~,true_list]=sort(this_V,1,'descend');    % Append outside option in finding max
    for sim =1:n_sim, w0=find(true_list(:,sim)==n_prog+1); true_list(w0:end,sim)=0; end
    tr=true(n_sim,1);
    for kk=1:3
        this_choice=true_list(kk,:);  % Which true choice are you looking at?
        tr = tr & (this_choice(:)==squeeze(chosen_list(ss,kk,:)));
        for pp=1:n_prog
            choice(kk).true(ss,pp)=mean(this_choice==pp);       % Probability pp is the true kk preferred program;
            choice(kk).report(ss,pp) = mean(chosen_list(ss,kk,:)==pp); % Probability pp is the reported in the  kk rank;
            choice(kk).report_data(ss,pp) = rank_data(ss,pp)==kk;   % Same above, in the data;
            
            % Manipulation : How often and in what position do you rank
            % your true choices.
            choice(kk).tr_any(ss,pp) = mean((this_choice'==pp).* squeeze(sum(chosen_list(ss,:,:)==pp,2))); % How often did you rank your kk-th choice and this choice was pp (in any position).
            choice(kk).tr_kk(ss,pp) = mean((this_choice'==pp).* squeeze(sum(chosen_list(ss,kk,:)==pp,2))); % How often did you rank your kk-th choice in the kk-the position and this choice was pp.

            %if kk==1 && this_choice(1)==pp && chosen_list(ss,kk,1)~=pp && sum(chosen_list(ss,:,1)==pp)>0
            %    kk
            %end
            % Assignment : How often do you get assigned to your true and
            % your reported choices.
            pp_assign=sum((chosen_list(ss,:,:)==pp).*camb_assign(ss,:,:),2); % probability of
                                                  % assignment to pp when ranked in any position.

            naiv_pp_assign = sum((naiv_list(ss,:,:)==pp).*naiv_assign(ss,:,:),2); % similarly for the naive list

            choice(kk).assign_any_tr(ss,pp) = mean((this_choice'==pp).*squeeze(pp_assign));  % probability of assignment to pp and pp is the kk-th true choice
            choice(kk).assign_rk(ss,pp) = mean(squeeze(chosen_list(ss,kk,:)==pp).*squeeze(pp_assign));  % probability of assignment to pp and pp is the kk-th ranked program
            choice(kk).assign_rk_tr(ss,pp) = mean(squeeze(pp_assign).*squeeze(chosen_list(ss,kk,:)==pp).*(this_choice'==pp));  % probability of assignment to pp, pp is your true kk choice and you ranked pp in the kk position;

            naiv(kk).assign(ss,pp) = mean(squeeze(naiv_pp_assign).*squeeze(naiv_list(ss,kk,:)==pp)); 
            % naifs always put the k-th school in position k

            % Manipulation : Ranked a school with priority
            choice(kk).choose_pp_priority(ss,pp) = mean(priority_data(ss,pp)*(this_choice'~=pp).*squeeze(sum(chosen_list(ss,kk,:)==pp,2))); % How often did you rank pp when you had priority as your kk-th choice even though it was not your kk-th choice
            choice(kk).choose_other_priority(ss,pp) = mean((~priority_data(ss,pp))*1*(this_choice'==pp).*squeeze(sum(chosen_list(ss,kk,:)~=pp,2))); % How often did you avoid ranking pp at kk-th choice because you did not have priority
        end
    end
    truthfulReport(ss)=mean(1*tr)*100;
end

%% Generate the table

% Redorder map
if override_method
    fid = fopen([est_type '_Choices_' method_id '.csv'], 'w');
else
    fid = fopen([est_type '_Choices.csv'], 'w');
end
    
% Handle to generate the table
gen_table = @(student_list, kk) 100*[mean(choice(kk).true(student_list,:),1); ...
                                 mean(choice(kk).report(student_list,:),1); ...
                                 mean(choice(kk).report_data(student_list,:),1); ...
                                 mean(choice(kk).tr_any(student_list,:),1); ...
                                 mean(choice(kk).tr_kk(student_list,:),1);...
                                 mean(choice(kk).assign_any_tr(student_list,:),1);...
                                 mean(choice(kk).assign_rk(student_list,:),1);...
                                 mean(choice(kk).assign_rk_tr(student_list,:),1);...
                                 mean(choice(kk).choose_pp_priority(student_list,:),1);... 
                                 mean(choice(kk).choose_other_priority(student_list,:),1);...
                                 mean(naiv(kk).assign(student_list,:),1)];
                             
rownames={'True','Reported','Reported Data','True in any rank', ...
          'True in this rank', 'True and Assigned','Ranked and Assigned',...
          'True Ranked and Assigned','Choose Due to Priority',...
          'Forego due to Priority','Naifs assigned in this choice'};

% Names of the columns in gen_table
tab_colname=cell(1,n_prog);  
for pp=1:n_prog
    if ~iscell(est.consolidate.name{pp});
        tab_colname{pp}=est.consolidate.name{pp};
    else
        tab_colname{pp}=est.consolidate.name{pp}{1};
    end
end
% Output the tables
fixedrows.name={'Students','Choice','Year'};
fprintf(fid,'.\n.\n');
printTable(fid,zeros(0,numel(tab_colname)),tab_colname,rownames,fixedrows,tab_cons);
fprintf(fid,'.\n.\n');
% All students, iterate choices
for yr=[0,2004,2005,2006,2007,2008]
    disp(yr)
which_students =  (data.year-yr)*yr==0;
for kk=1:3
    fixedrows.value={'All',[num2str(kk) '-th Choice'],num2str(yr)};
    table=gen_table(which_students & student_set,kk);
    printTable(fid,table,tab_colname,rownames,fixedrows,tab_cons,1);
    fprintf(fid,'.\n.\n');
end


% Free/Reduced lunch students, iterate choices
which_students =  ~logical(data.paidlunch)  & (data.year-yr)*yr==0;
for kk=1:3
    fixedrows.value={'Freelunch',[num2str(kk) '-th Choice'],num2str(yr)};
    table=gen_table(which_students & student_set,kk);
    printTable(fid,table,tab_colname,rownames,fixedrows,tab_cons,1);
    fprintf(fid,'.\n.\n');
end

% Paidlunch students, iterate choices
which_students =  logical(data.paidlunch)  & (data.year-yr)*yr==0 & est.esample;
for kk=1:3
    fixedrows.value={'Paidlunch',[num2str(kk) '-th Choice'],num2str(yr)};
    table=gen_table(which_students & student_set,kk);
    printTable(fid,table,tab_colname,rownames,fixedrows,tab_cons,1);
    fprintf(fid,'.\n.\n');
end
% Rationalizes students, iterate choices
%which_students =  est.stud.rationalized  & (data.year-yr)*yr==0;
%for kk=1:3
%    fixedrows.value={'Rationalized',[num2str(kk) '-th Choice'],num2str(yr)};
%    table=gen_table(which_students & student_set,kk);
%    printTable(fid,table,tab_colname,rownames,fixedrows,tab_cons,1);
%    fprintf(fid,'.\n.\n');
%
%end

end

fclose(fid);

%% Generate the truthful table:
student_set = 1;
fid = fopen([est_type '_TruthfullList.csv'], 'w');
fprintf(fid,'Students,Year,Truthful\n');
                  
% Output the tables
% All students, iterate choices
for yr=[0,2004,2005,2006,2007,2008]
    which_students = (data.year-yr)*yr==0;
    fprintf(fid,'All,%d,%3.2f\n',yr,mean(truthfulReport(which_students & student_set)));
    % Free/Reduced lunch students, iterate choices
    which_students = ~logical(data.paidlunch)  & (data.year-yr)*yr==0;
    fprintf(fid,'Freelunch,%d,%3.2f\n',yr,mean(truthfulReport(which_students & student_set)));
    % Paidlunch students, iterate choices
    which_students = logical(data.paidlunch)  & (data.year-yr)*yr==0;
    fprintf(fid,'Paidlunch,%d,%3.2f\n',yr,mean(truthfulReport(which_students & student_set)));
    % Rationalizes students, iterate choices
    %which_students = est.stud.rationalized  & (data.year-yr)*yr==0;
    %fprintf(fid,'Rationalized,%d,%3.2f\n',yr,mean(truthfulReport(which_students & student_set)));
end
fclose(fid);

