%% Set up
%est_type = 'currentyear1_ts';
%override_method = false; method_id = 'current_year_ts'; % change it to current_year_ts
%run_prep_da_vs_camb = true; % Run it once after a each DA run.
%% Use current_year to compute actual assignment probabilities;
%asgn_lott = 'current_year_ts'; 




%% Load stuff[]
if truthful_behavior, tr = '_truthful_'; else tr='_'; end
file_prefix = ['../../da_counter/mat/' est_type tr];
est=load(['../../estimation/' est_type '/master']);
data=load('../../estimation/data');

% Draws
switch draws_from
    case 'bootstrap', draws = bootstrap_draws;
    case 'posterior', draws = posterior_draws;
    otherwise draws = input('How many draws? :');
end

bs=struct(); bs.smpl=[];
bs_accum = 0; bb = 0;
while bs_accum < draws && bb <1000
    bb = bb+1;
    fname = [file_prefix num2str(bb) '.mat'];
    if exist(fname,'file')==2 & bs_accum<draws
        bs_accum = bs_accum +1;
        this_draw=load(fname);
        if ~isfield(this_draw,'pref_da_camb')
            this_draw.pref_da_camb=cell(numel(this_draw.V_da),1);
            for yy = 1:numel(this_draw.V_da)
                this_draw.pref_da_camb{yy}=mean(this_draw.V_da{yy}-this_draw.V_camb{yy},2);
            end
        end
        for yy = 1:numel(this_draw.pref_da_camb)
            bs.pref_da_camb{yy}(:,bs_accum)=this_draw.pref_da_camb{yy};
            bs.frac_pref_da{yy}(:,bs_accum)=this_draw.frac_pref_da{yy};
            bs.frac_indiff{yy}(:,bs_accum) = this_draw.frac_indiff{yy};
            bs.frac_indiff{yy}(:,bs_accum) = this_draw.frac_indiff{yy};
            bs.frac_je{yy}(:,bs_accum)     = this_draw.frac_je{yy};
        end
        this_draw =load(['../../est_lotteries/bootstrap/current_year_ts' num2str(bb)],'smpl');
        bs.smpl(:,bs_accum) = this_draw.smpl;
    end
end
fprintf('Found %d %s draws out of %d required (Checked %d)\n', bs_accum,draws_from,draws,bb)       
if bs_accum ==0, bs.smpl=[]; end
% Point estimate
load([file_prefix '0']);


n_choices=est.n_progs-1;
student_set = ismember(data.year,unique(data.year(est.esample)));
 % In next versions this should be taken care of in run_da_posterior; The
 % student's dimension should not change, students shouldn't be dividided
 % into cells.


if strcmp(est_type,'last_year'),
    student_set = ~(data.year ==2004 | data.year ==2007);
end
%% Compute stuff
n_sim=size(V_vec{1},3);
years=unique(data.year(est.esample));

%% Run prep files: compute utility differences between Cambridge and DA
if ~run_prep_da_vs_camb && ~exist(['prep' est_type tr '.mat'],'file')
    fprintf('Ignoring run_prep_da_vs_camb = false because prep file is missing\n')
    run_prep_da_vs_camb = true;
end

if run_prep_da_vs_camb
    da_p_choice=cell(size(V_vec)); % Distribution over choices
    camb_p_choice=cell(size(V_vec)); % Distribution over choices
    pref_da_camb=cell(size(V_vec)); % DA vs CAMB preferences
    
    for ii=1:size(V_vec,2)
       da_p_choice{ii}=zeros(size(V_vec{ii},1),size(V_vec{ii},3),n_choices); 
       camb_p_choice{ii}=zeros(size(V_vec{ii},1),size(V_vec{ii},3),n_choices); 
       pref_da_camb{ii}=zeros(size(V_vec{ii},1),size(V_vec{ii},3));
    end

    %student_set=false(n_stu,1);
    for yy=1:numel(years)
        disp(num2str(years(yy)));

        % Which students
        which_students=find(all([data.year==years(yy)],2));        
        if any(student_set(which_students))
        n_stu=size(V_vec{yy},1);    
        assert(n_stu==size(which_students,1));

        for ii=1:n_stu
            v_camb_temp=zeros(1,n_sim);
            for bb=1:n_sim
                % Sort the utility vector
                Vo=[V_vec{yy}(ii,:,bb)];
                v=sort(unique(Vo),'descend');

                % Find the location of the choices
                for cc=1:n_choices                
                    if v(cc)>0
                        % Assignment to cc-th choice - DA. Which school
                        % is chosen? (Different from tab_da)
                        which_program=[V_vec{yy}(ii,:,bb)]==v(cc);
                        which_school=unique(est.consolidate.map(which_program));
                        assert(numel(which_school)==1);
                        da_p_choice{yy}(ii,bb,cc)=da_assign{yy}(ii,which_school,bb);                            

                        % Compute the report
                        which_program=find(V_vec{yy}(ii,:,bb)==v(cc));
                        %% % Assignment to cc-th choice - Cambridge
                        cc_loc=ismember(chosen_list{yy}(ii,:,bb),which_program);                        
                        if any(cc_loc)                            
                            camb_p_choice{yy}(ii,bb,cc)=sum(camb_assign{yy}(ii,cc_loc,bb));
                        end                             
                        %%
                    end                
                end                           
            end

            pref_da_camb{yy}(ii,:)=V_da{yy}(ii,:)-V_camb{yy}(ii,:);

        end
        end
    end
    save(['prep' est_type tr '.mat'],'da_p_choice','camb_p_choice','pref_da_camb')
else
    load(['prep' est_type tr '.mat'],'da_p_choice','camb_p_choice','pref_da_camb')
end

%% Compute Assignmnet probability to Proximity and Sibling Priority
for yy=1:numel(years)
    da_prob = mean(da_assign{yy},3);
    camb_prob = zeros(size(da_prob));
    for ii= 1:length(camb_prob)
        for ss = 1:n_sim
            this_list = chosen_list{yy}(ii,:,n_sim);
            this_list = est.consolidate.map(this_list(~isnan(this_list) & this_list>0));
            camb_prob(ii,this_list) = camb_prob(ii,this_list) + (1/n_sim)*camb_assign{yy}(ii,1:length(this_list),ss);        
        end
    end
    
    which_students=all([data.year==(2003+yy)],2);        
end

% Fix missing years in last_year
all_years = min(data.year):max(data.year);
if numel(all_years)>numel(years)
    for yy = numel(all_years):-1:1;
        cell_index=find(years==all_years(yy));
        if numel(cell_index)>0
            da_p_choice{yy} = da_p_choice{cell_index};
            camb_p_choice{yy} = camb_p_choice{cell_index};
            pref_da_camb{yy} = pref_da_camb{cell_index};
            JE_camb{yy}      = JE_camb{cell_index};
            bs.pref_da_camb{yy} = bs.pref_da_camb{cell_index};
            bs.frac_pref_da{yy} = bs.frac_pref_da{cell_index};
            bs.frac_indiff{yy} =  bs.frac_indiff{cell_index};
            bs.frac_indiff{yy} =  bs.frac_indiff{cell_index};
            bs.frac_je{yy}     =  bs.frac_je{cell_index};
        else
            n_stu = sum(data.year == all_years(yy));
            da_p_choice{yy} = nan(n_stu,size(da_p_choice{1},2),size(da_p_choice{1},3));
            camb_p_choice{yy} = nan(n_stu,size(da_p_choice{1},2),size(da_p_choice{1},3));
            pref_da_camb{yy}= nan(n_stu,size(da_p_choice{1},2));
            JE_camb{yy}     = nan(n_stu,size(da_p_choice{1},2));
            bs.pref_da_camb{yy} = nan(n_stu,bs_accum);
            bs.frac_pref_da{yy} = nan(n_stu,bs_accum);
            bs.frac_indiff{yy}  = nan(n_stu,bs_accum);
            bs.frac_je{yy}      = nan(n_stu,bs_accum);
        end
    end
end
%% Create the tables
da_p_choice=cell2mat(da_p_choice');
camb_p_choice=cell2mat(camb_p_choice');
pref_da_camb=cell2mat(pref_da_camb');
JE_camb=cell2mat(JE_camb');
bs.pref_da_camb= cell2mat(bs.pref_da_camb');
bs.frac_pref_da= cell2mat(bs.frac_pref_da');
bs.frac_indiff= cell2mat(bs.frac_indiff');
bs.frac_je = cell2mat(bs.frac_je');

% Averages across simulations
mean_da_p_choice=mean(da_p_choice,2);
mean_camb_p_choice=mean(camb_p_choice,2);


mean_u_diff_da_camb=mean(pref_da_camb,2);
frac_pref_da=mean(pref_da_camb>1e-5,2);
frac_indiff=mean(abs(pref_da_camb)<=1e-5,2);
frac_pref_camb=1 - frac_pref_da - frac_indiff;
bs.frac_pref_camb = 1 - bs.frac_pref_da - bs.frac_indiff;


%% Write to file
% Handle to generate the table
sz = @(mat) size(mat,1);
test_dim = [sz(mean_da_p_choice) sz(mean_camb_p_choice) sz(mean_u_diff_da_camb) sz(pref_da_camb) ...
sz(JE_camb)];
if std(test_dim)> 0
    test_dim
    error('Matrices have different number of rows')
end 

meanNaN = @(x) mean(x(~isnan(x)));
gen_table = @(student_list) full([100* ...
                    squeeze(mean(mean_da_p_choice(student_list,:,:)))', ...
                    100*squeeze(mean(mean_camb_p_choice(student_list,:,:)))', ...
                    mean(mean_u_diff_da_camb(student_list)), ...
                    std(bootstrap_se(bs,'pref_da_camb',student_list)),...     sqrt((mean(mean_u_diff_da_camb_sq(student_list)) - mean(mean_u_diff_da_camb(student_list))^2)/sum(student_list>0)), ...
                    std(mean_u_diff_da_camb(student_list)), ...
                    100*mean(frac_pref_da(student_list)), ...
                    100*std(bootstrap_se(bs,'frac_pref_da',student_list)), ...
                    100*mean(frac_indiff(student_list)), ...
                    100*std(bootstrap_se(bs,'frac_indiff',student_list)), ...
                    100*mean(frac_pref_camb(student_list)), ...
                    100*std(bootstrap_se(bs,'frac_pref_camb',student_list)), ...
                    100*mean(meanNaN(JE_camb(student_list,:))),...
                    100*std(bootstrap_se(bs,'frac_je',student_list)), ...   %100*mean(meanNaN(justified_envy(student_list,:).*Nv(student_list,:)))/mean(mean(Nv(student_list,:))),...
                    mean(bootstrap_se(bs,'pref_da_camb',student_list)) ]);

flat = @(x) x(:);
gen_u_plot_kernel =  @(student_list) ksdensity(flat(pref_da_camb(student_list,:)),linspace(-0.5,0.5,10000));
gen_u_plot_ecdf =  @(student_list) ecdf(flat(pref_da_camb(student_list,:)));
tab.colname{1}='Students';
for ii=1:n_choices
   tab.colname{ii+1}=['P(' num2str(ii) '-th Choice DA)'];
   tab.colname{ii+1+n_choices}=['P(' num2str(ii) '-th Choice Camb)'];    
end
add_colsname={'Mean V(DA)-V(CAMB)', 'SEM(V(DA)-V(CAMB))', 'Std V(DA)-V(CAMB)','Frac V(DA)>V(CAMB)', 'SE Frac V(DA)>V(CAMB)','Frac V(DA)=V(CAMB)','SE Frac V(DA)=V(CAMB)','Frac V(DA)<V(CAMB)', 'SE Frac V(DA)<V(CAMB)','Justified Envy','JE_Behavior','Bhv_esample','Behavior','Mean Bootstrap Samples'};add_colsname={'Mean V(DA)-V(CAMB)', 'SEM(V(DA)-V(CAMB))', 'Std V(DA)-V(CAMB)','Frac V(DA)>V(CAMB)', 'SE Frac V(DA)>V(CAMB)','Frac V(DA)=V(CAMB)','SE Frac V(DA)=V(CAMB)','Frac V(DA)<V(CAMB)', 'SE Frac V(DA)<V(CAMB)','Justified Envy','SE Justified Envy','Mean Bootstrap Samples'};
tab.colname(1+2*n_choices+(1:numel(add_colsname)))=add_colsname;
n_cols_so_far=size(tab.colname,2);

 % Populate rows
tab.rowname={'All Students','Paid Lunch','Free/Reduced Lunch'};

% All students
student_set = ismember(data.year,years);  
student_list=student_set;
tab.rows=gen_table(student_list);
al.k = gen_u_plot_kernel(student_list);
[al.ecdf,al.x] = gen_u_plot_ecdf(student_list);

% Paid lunch
student_list=logical(data.paidlunch) & student_set;
tab.rows=[tab.rows;gen_table(student_list)];
pl.k = gen_u_plot_kernel(student_list);
[pl.ecdf,pl.x] = gen_u_plot_ecdf(student_list);

% Free Lunch
student_list=~logical(data.paidlunch) & student_set;
tab.rows=[tab.rows;gen_table(student_list)];
[fl.k,xi] = gen_u_plot_kernel(student_list);
[fl.ecdf,fl.x] = gen_u_plot_ecdf(student_list);

% Generate Table
filename = [est_type tr 'da.csv'];
fid = fopen(filename, 'w');
for ii=1:size(tab.colname,2)
    fprintf(fid,tab.colname{ii});
    if ii<size(tab.colname,2)
        fprintf(fid,',');
    else
        fprintf(fid,'\n');
    end
end
for ii=1:size(tab.rowname,2)
    fprintf(fid,tab.rowname{ii});
    for jj=1:size(tab.rows,2)
        fprintf(fid,',%3.3f',tab.rows(ii,jj));
    end
    fprintf(fid,'\n');
end

fclose(fid);


fprintf('%s Saved.\n',filename)


% Generate Plots

clean_text = @(x) strrep(x,'_',' ');

% ECDF
plot(al.x,al.ecdf,'-k');
hold on
plot(pl.x,pl.ecdf,'-r');
plot(fl.x,fl.ecdf,'-b');
hold off
legend('All Students','Paid Lunch','Free Lunch');
title(clean_text([est_type tr]))
saveas(gcf,['figs/' est_type tr 'ecdf.png']);
lim = axis; axis([-1e-4 1e-4 lim(3) lim(4)]);
saveas(gcf,['figs/' est_type tr 'zoom.png']);
cla;
% Kernel
cla;
plot(xi,al.k,'-k');
hold on
plot(xi,pl.k,'-r');
plot(xi,fl.k,'-b');
hold off
legend('All Students','Paid Lunch','Free Lunch');
title(clean_text([est_type tr]))
saveas(gcf,['figs/' est_type tr 'kern.png']);

% Box Plot
boxplot(pref_da_camb(:),flat(repmat(data.paidlunch,1,size(pref_da_camb,2))),'Labels',{'free lunch','paid lunch'});
title(clean_text([est_type tr]))
saveas(gcf,['figs/' est_type tr 'box.png']);







