

* Descriptive Statistics

use /proj/cambridge/lottery_file,clear
gen asg1 = (asgsch == sch1) & asgsch ~= .
gen asg2 = asgsch == sch2 & asgsch ~= .
gen asg3 = asgsch == sch3 & asgsch ~= .
gen asg0 = 1 - asg1 - asg2 - asg3

tempfile table1
preserve
di "The following table breakes down the assignments at each step: 1st, 2nd, 3rd, no assignment"
table year, c(freq mean asg1 mean asg2 mean asg3 mean asg0) replace
ren table1 freq
ren table2 first_rnd
ren table3 second_rnd
ren table4 third_rnd
ren table5 unassigned
sort year
save `table1'
restore

preserve
di "The following table breakes down the assignments at each step: 1st, 2nd, 3rd, no assignment"
table year if paidlunch, c(freq mean asg1 mean asg2 mean asg3 mean asg0) replace
ren table1 freq_paid
ren table2 first_rnd_paid
ren table3 second_rnd_paid
ren table4 third_rnd_paid
ren table5 unassigned_paid
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore

preserve
di "The following table breakes down the assignments at each step: 1st, 2nd, 3rd, no assignment"
table year if ~paidlunch, c(freq mean asg1 mean asg2 mean asg3 mean asg0) replace
ren table1 freq_free
ren table2 first_rnd_free
ren table3 second_rnd_free
ren table4 third_rnd_free
ren table5 unassigned_free
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore

preserve
table year, c(freq mean prox1 mean prox2 mean prox3) replace
ren table1 freq 
ren table2 proxp1
ren table3 proxp2
ren table4 proxp3
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore
preserve
gen rankedp0= prox1 + prox2 + prox3 <0.5
gen rankedp1= prox1 + prox2 + prox3 <1.5
gen rankedp2= prox1 + prox2 + prox3 <2.5
table year, c(freq mean rankedp0 mean rankedp1 mean rankedp2) replace
ren table1 freq 
ren table2 rankedp0
ren table3 rankedp1
ren table4 rankedp2
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore

* Language and Ethnicity
preserve
gen hlang_en =  hlang == 100 | hlang ==267
gen hlang_sp =  hlang == 7
gen hlang_pg =  hlang == 6
table year, c(mean hlang_en mean hlang_sp mean hlang_pg) replace
ren table1 English 
ren table2 Spanish
ren table3 Portuguese
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore
preserve
gen eth_bk = eth == 2
gen eth_as = eth == 3
gen eth_hs = eth == 5
gen eth_wh = eth == 6
table year, c(mean eth_wh mean eth_bk mean eth_as mean eth_hs ) replace
ren table1 White
ren table2 Black
ren table3 Asian
ren table4 Hispanic
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore


* Distances
preserve
gen nrk = (sch1~=.)+(sch2~=.)+(sch3~=.)
gen d_closest = 99
gen d_mean = 0
gen d_mean_ranked = 0
gen d_assigned = 0
levelsof sch1, local(schools)

foreach sc of local schools {
	qui replace d_closest = min(d_closest,distance`sc')
	qui replace d_mean = d_mean + distance`sc' /wordcount("`schools'")
	qui replace d_mean_ranked = d_mean_ranked + distance`sc'/nrk if sch1==`sc' | sch2==`sc' | sch3==`sc'
	qui replace d_assigned = distance`sc' if asgsch==`sc' 
}
table year, c(mean d_closest mean d_mean mean d_mean_ranked mean d_assigned ) replace
ren table1 d_closest
ren table2 d_mean
ren table3 d_mean_ranked
ren table4 d_assigned
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore


preserve
gen d_rnk1 = 0
levelsof sch1, local(schools)

foreach sc of local schools {
	qui replace d_rnk1 = distance`sc' if sch1==`sc' 
}
table year, c(mean d_rnk1) replace
ren table1 d_rnk1
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore



* Number of programs ranked
preserve
gen nrk = (sch1~=.)+(sch2~=.)+(sch3~=.)
gen nrk1 = nrk==1
gen nrk2 = nrk==2
gen nrk3 = nrk==3
table year, c(mean nrk1 mean nrk2 mean nrk3 ) replace
ren table1 nrk1
ren table2 nrk2
ren table3 nrk3
sort year
merge year using `table1'
sort year
drop _merge
save `table1', replace
restore

* Sibling priority

preserve
table year, c(freq mean sib1 mean sib2 mean sib3 mean paidlunch) replace
ren table1 freq 
ren table2 siblp1
ren table3 siblp2
ren table4 siblp3
ren table5 prop_paidlunch
sort year
merge year using `table1'
drop _merge
sort year
save `table1', replace
restore
preserve
use /proj/cambridge/capacity_file,clear
sort year sch prog
gen newsch = 0
replace newsch = 1 if sch[_n]!=sch[_n-1]
gen truecap = tot_capacity * newsch
table year, c(freq sum newsch sum truecap) replace
di "Note: OLA, portuguese appears as a different school, do we want to think about it as a program in King Open instead, probably we should be consistent with Amigos bilingual".
ren table1 programs 
ren table2 schools
ren table3 seats

sort year
merge year using `table1'
drop _merge
ren freq students
save `table1', replace



desc



outsheet _all using /proj/cambridge/tables/DescriptiveStatistics/AllStats.csv, comma replace
outsheet year school programs seats students prop_paidlunch prop_paidlunch  White Black Asian Hispanic English Spanish Portuguese d_closest d_mean using /proj/cambridge/tables/DescriptiveStatistics/AggregateQuantities.csv, comma replace
outsheet year *_rnd unassigned d_rnk1 d_mean_ranked d_assigned  nrk* rankedp* using /proj/cambridge/tables/DescriptiveStatistics/RoundOfAssignment.csv, comma replace

