% Does the RD plots.
% Plots (a binscatter of) Y_{ij} against X_{ij}
% * Y_{ij} = whether school j was a top rank for student i
% * X_{ij} = distance of i to j, minus the mean of the second and third
%   distance of i to j
%   - if the distance between 2 and 3 is 0, I use the distance between 3
%     and 4
% 
% NOTES: We group Amigos into one school, same for Tobin, same for KingOp

% clear;

% load ../../estimation/data;
% years = 2004:2008; % Which years to include


% % Directions:
% % * benchmark_r1 : use_ranks = 1, nbins = [5 10], order = 3, keep_dranks =
  % % 1:12, dmix = 2:3, drop_bad_benchmark = 0;
% % * benchmark_r2 : use_ranks = 2 ...
% % * benchmark_r3 : use_ranks = 3 ...
% % * placebo_12 : use_ranks = 1, nbins = [5 5], order = 3, keep_dranks =
  % % 1:2, dmix = 1:2, drop_bad_benchmark = 1;
% % * just_23 : use_ranks = 1, nbins = [5 5], order = 3, keep_dranks =
  % % 2:3, dmix = 2:3, drop_bad_benchmark = 1;
% % * placebo_3plus: use_ranks = 1, nbins = [5 5], order = 2, keep_dranks =
  % % 3:12, dmix = 4:5, drop_bad_benchmark = 1;

% use_ranks = 1; % What it means to be a "top rank"  (which ranks get Y_ij = 1)
% nbins = [5 10]; % Try [4 6] for the schools; Try [5 8] for aggregates -- number of bins to left and right of the cutoff
% order = 3; % Order of polynomial fit
% keep_dranks = 1:12; % Which distances we keep (keep ties as well)
% dmix = 1:2; % Which distances to mix; 2:3 is standard
% drop_bad_benchmark = 0; % If distances are tied, what do we do?  0 -- mix 3 and 4 if 2 and 3 are tied, 1 -- drop these guys
% paidlunches = 0; % Which types of students to keep 0:1 for both


% which_schools = [10 13 15 25 20 23 28]; % Competitive schools are [10 13 15 25 20 23 28] 
% which_schools = [17 18 21 24 27 29]; % non-competitive schools are [17 18 21 24 27 29]
% which_schools = [10 13 15 25 20 28 17 18 21 23 24 27 29]; % all schools


% Pre-set parameters
gray = [0.65, 0.65, 0.65];
vec_op = @(ZZ)(ZZ(:));

tnbins = sum(nbins);
full_schools = rank.code;
full_schools(end) = 18000; % Combine King OLA and King Open
school_num = floor(full_schools / 1000);
[school_list, index, back_index] = unique(school_num);
nschools = length(school_list);
nstudents = size(dists, 1);

aggregated_distances = dists(:, index);
names = rank.name(index);



% Let's just do this using a "for" loop.  It'll be quick
Y_matrix = zeros(nstudents, nschools);
for sch = 1:nschools
    relevant_cols = (back_index == sch);  
    for stu = 1:nstudents
        if any(ismember(rank.data(stu, relevant_cols), use_ranks))
            Y_matrix(stu, sch) = 1;
        end
    end
end

% Now modify the distances
[s, ws] = sort(aggregated_distances, 2);
benchmark = mean(s(:, dmix), 2);
where_bad_benchmark = abs(s(:, dmix(2)) - s(:, dmix(1))) < 1e-6;
if drop_bad_benchmark == 0
    alternate_benchmark = mean(s(:, (dmix+1)), 2);
    benchmark(where_bad_benchmark) = alternate_benchmark(where_bad_benchmark);
    all_year = year;
else
    Y_matrix = Y_matrix(~where_bad_benchmark, :);
    aggregated_distances = aggregated_distances(~where_bad_benchmark, :);
    benchmark = benchmark(~where_bad_benchmark);
    ws = ws(~where_bad_benchmark, :);
    all_year = year(~where_bad_benchmark, :);
    paidlunch = paidlunch(~where_bad_benchmark);
end

% limit to the schools of interest
these_schools = ismember(school_list,which_schools);
aggregated_distances = aggregated_distances(:,these_schools);
Y_matrix = Y_matrix(:,these_schools);

modified_distances = bsxfun(@minus, aggregated_distances, benchmark);
nschools = size(Y_matrix, 2);

% get rid of the years you don't want and the students you don't want
okay_students = ismember(all_year, years) & ismember(paidlunch,paidlunches);
this_md = modified_distances(okay_students, :);
Y_matrix = Y_matrix(okay_students, :);

% Create dummies
nstudents = size(this_md, 1);
dummies = zeros(nstudents*nschools, nschools);
for i = 1:nschools
    temp_mat = zeros(nstudents, nschools);
    temp_mat(:, i) = 1;
    dummies(:, i) = vec_op(temp_mat);
end


%%%%%%% A. Imbens-Kalyaraman Optimal Bandwidth 
fprintf('**** %s ****\n',sp)
% Set up the data
which_students = ~any(isnan(this_md),2);
this_md_vec_test = vec_op(this_md(which_students,:));
Y_matrix_vec_test = vec_op(Y_matrix(which_students,:));
controls = dummies(repmat(which_students,nschools,1),2:end); % Drop one, we always include a constant anyways.
cluster = vec_op(repmat([1:sum(which_students)]',1,size(Y_matrix,2)));


% Cluster Robust SE;
[Y,X,h_opt,incl]=rd_optbandwidth_ps(Y_matrix_vec_test,this_md_vec_test,(this_md_vec_test>0),controls,0,0,0);
fprintf('Bandwith: %3.3f; Included Observations: %d\n',h_opt,sum(incl));
fancy_regress(Y,X,cluster(incl))


%%%%%% B. Now do the scatterplots


% Now convert all the distances we don't care about to NaNs
if length(keep_dranks) < nschools
    for i = 1:nstudents
        good_distances = aggregated_distances(i, ws(i, keep_dranks));
        
        % This is a hack
        good_distances = round(1000 * good_distances);
        my_distances = round(1000 * aggregated_distances(i, :));
        where_in_row = ismember(my_distances, good_distances);
        
        this_md(i, ~where_in_row) = NaN;
        Y_matrix(i, ~where_in_row) = NaN;        

        % % Old code -- doesn't deal with Amigos and MLK
        % modified_distances(i, ws(i, ignore_dranks)) = NaN;
        % Y_matrix(i, ws(i, ignore_dranks)) = NaN;
    end
end



% Now do the regression
Y_matrix_vec = vec_op(Y_matrix);
this_md_vec = vec_op(this_md);

low_x = nanmin(this_md_vec);
high_x = nanmax(this_md_vec);

[bin_locs, matrix] = make_bins(this_md_vec, 0, nbins);

[b, bint] = regress(Y_matrix_vec, matrix);
[b_dum, bint_dum] = regress(Y_matrix_vec, [matrix, dummies(:, 2:end)]);

% Now need to recenter the dummies
old_mean = nanmean(Y_matrix_vec);
new_mean = nanmean(Y_matrix_vec - dummies(:, 2:end) * b_dum(tnbins+1:end));
b_dum_new = b_dum + (old_mean - new_mean);

%%% Also need the cubic fit -- replaced with local linear
p_low = polyfit(this_md_vec(this_md_vec < 0), Y_matrix_vec(this_md_vec < 0), order);
p_high = polyfit(this_md_vec(this_md_vec >= 0), Y_matrix_vec(this_md_vec >= 0), order);

% Use local linear regressions for the fit
r_low = ksrlin(this_md_vec(this_md_vec < 0), Y_matrix_vec(this_md_vec < 0));
r_high = ksrlin(this_md_vec(this_md_vec >= 0), Y_matrix_vec(this_md_vec >= 0));


%%% Also do a cubic fit with dummies - not used
power_mat = zeros(nstudents*nschools, order+1);
for i = 0:order
    power_mat(:, i+1) = this_md_vec .^ (order-i);
end
mat_for_poly = [power_mat, dummies(:, 2:end)];
dummy_p_low_all = regress(Y_matrix_vec(this_md_vec < 0), mat_for_poly(this_md_vec < 0, :));
dummy_p_low = dummy_p_low_all(1:order+1)';

dummy_p_high_all = regress(Y_matrix_vec(this_md_vec > 0), mat_for_poly(this_md_vec > 0, :));
dummy_p_high = dummy_p_high_all(1:order+1)';

% Now recenter the cubic fit
old_mean_low = mean(Y_matrix_vec(this_md_vec < 0));
new_mean_low = mean(Y_matrix_vec(this_md_vec < 0) - dummies(this_md_vec < 0, 2:end) * dummy_p_low_all(order+2:end));

old_mean_high = mean(Y_matrix_vec(this_md_vec > 0));
new_mean_high = mean(Y_matrix_vec(this_md_vec > 0) - dummies(this_md_vec > 0, 2:end) * dummy_p_high_all(order+2:end));

dummy_p_low(end) = dummy_p_low(end) + (old_mean_low - new_mean_low);
dummy_p_high(end) = dummy_p_high(end) + (old_mean_high - new_mean_high);

% Nikhil -- Change marker size here
errorbar(bin_locs, b_dum_new(1:tnbins), bint_dum(1:tnbins, 2) - b_dum(1:tnbins), 'ok', 'MarkerFaceColor', gray, 'MarkerSize', 4, 'MarkerEdgeColor', gray, 'LineWidth', 1.2, 'Color', gray); hold on;
errorbar(bin_locs, b(1:tnbins), bint(1:tnbins, 2) - b(1:tnbins), 'ok', 'MarkerFaceColor', 'k', 'MarkerSize', 4, 'MarkerEdgeColor', 'k', 'LineWidth', 1.2); 

xx = xlim; % Deliberately in this order to avoid extreme extrapolation
axis([low_x, high_x, ylim]);

low_xvals = linspace(xx(1), 0, 1000);
low_polyvals = polyval(dummy_p_low, low_xvals);
high_xvals = linspace(0, xx(2), 1000);
high_polyvals = polyval(dummy_p_high, high_xvals);

% Cubic fits are not used
%plot(low_xvals, low_polyvals, '--', 'Color', gray, 'LineWidth', 1.2); % Nikhil -- Change width of gray line here
%plot(high_xvals, high_polyvals, '--', 'Color', gray, 'LineWidth', 1.2);
plot([0, 0], ylim, ':k', 'LineWidth', 1);

low_xvals = linspace(xx(1), 0, 1000);
low_polyvals = polyval(p_low, low_xvals);
high_xvals = linspace(0, xx(2), 1000);
high_polyvals = polyval(p_high, high_xvals);

%plot(low_xvals, low_polyvals, '--k', 'LineWidth', 1); % Nikhil -- Change width of black line here
plot(r_low.x, r_low.f, '--k', 'LineWidth', 1);
%plot(high_xvals, high_polyvals, '--k', 'LineWidth', 1);
plot(r_high.x, r_high.f, '--k', 'LineWidth', 1);
yy = ylim;

% Overwrite axis when appropriate
if exist('overwrite_y_lim','var') && numel(overwrite_y_lim), yy(2)=overwrite_y_lim; end
axis([xlim 0 yy(2)]);
plot([0, 0], ylim, ':k', 'LineWidth', 1);

hold off;

if all(dmix == 2:3)
    xlabel('Distance from Proximity Boundary (miles)');
elseif all(dmix == 1:2)
    xlabel('Distance from Midpoint of Closest Two Schools (miles)');
else
    xlabel('Distance from Placebo Boundary (miles)');
end
    
if use_ranks == 1
    ylabel('Probability Ranked 1st');
elseif use_ranks == 2
    ylabel('Probability Ranked 2nd');
elseif use_ranks == 3
    ylabel('Probability Ranked 3rd');
end
listOffonts = listfonts;
k = 9;  
set(findall(gcf,'type','axes'),'fontName',listOffonts{k},'fontSize',14)
set(findall(gcf,'type','text'),'fontName',listOffonts{k},'fontSize',14) 

saveas(gcf,sp,'jpeg')
fprintf('Plot Saved\n',sp)



