% Does the RD plots.
% Plots (a binscatter of) Y_{ij} against X_{ij}
% * Y_{ij} = whether school j was a top rank for student i
% * X_{ij} = distance of i to j, minus the mean of the second and third
%   distance of i to j
%   - if the distance between 2 and 3 is 0, I use the distance between 3
%     and 4
% 
% NOTES: We group Amigos into one school, same for Tobin, same for KingOp


%% Runs all the rd-specifications

tasks.rd_specs = {'benchmark_r1_all',...        fig I.i
                  'benchmark_r2_all',...        fig I.ii
                  'benchmark_r3_all',...        fig I.iii
                  'competitive_r1_all',...      fig I.iv
                  'noncompetitive_r1_all',...   fig I.v
                  'placebo12_r1_all',...        fig I.vi
                  'benchmark_r1_paidlunch',...  fig D.I.ii
                  'benchmark_r2_paidlunch','benchmark_r3_paidlunch',...
                  'competitive_r1_paidlunch','noncompetitive_r1_paidlunch','placebo12_r1_paidlunch',...
                  'benchmark_r1_freelunch',...  fig D.I.iii
                  'benchmark_r2_freelunch','benchmark_r3_freelunch',...
                  'competitive_r1_freelunch','noncompetitive_r1_freelunch','placebo12_r1_freelunch',...
                  'just23_r1_all',...           fig D.I.i (not in table V)
                  'placebo3plus_r1_all'...      fig D.I.iv (not in table V)
                  };


% Loop over specifications
for sp = tasks.rd_specs
    sp = sp{1};
    
    % Default values
    use_ranks = 1; % What it means to be a "top rank"  (which ranks get Y_ij = 1)
    nbins = [5 10]; % Try [4 6] for the schools; Try [5 8] for aggregates -- number of bins to left and right of the cutoff
    order = 3; % Order of polynomial fit
    keep_dranks = 1:12; % Which distances we keep (keep ties as well)
    dmix = 2:3; % Which distances to mix; 2:3 is standard
    drop_bad_benchmark = 0; % If distances are tied, what do we do?  0 -- mix 3 and 4 if 2 and 3 are tied, 1 -- drop these guys
    spec_details = strsplit(sp,'_');
    which_schools = [10 13 15 25 20 28 17 18 21 23 24 27 29]; % all schools
    paidlunches = 0:1;
    overwrite_y_lim = [];
    % Specification type
    switch spec_details{1}
        case 'placebo12',
            nbins = [5 5]; 
            keep_dranks = 1:2;
            dmix = 1:2;
            % drop_bad_benchmark = 1;
        case 'placebo3plus'
            nbins = [5 5]; 
            order = 2; 
            keep_dranks =  3:12;
            dmix = 4:5;
            drop_bad_benchmark = 1;
        case 'just23',
            nbins = [5 5];
            keep_dranks = 2:3;
            dmix = 2:3;
            drop_bad_benchmark = 1;        
        case 'competitive'
            which_schools = [10 13 15 25 20 23 28]; % Competitive schools are [10 13 15 25 20 23 28] 
        case 'noncompetitive'
            which_schools = [17 18 21 24 27 29]; % non-competitive schools are [17 18 21 24 27 29]
    end
    % Rank?
    switch spec_details{2}
        case 'r2'
            use_ranks = 2;
        case 'r3'
            use_ranks = 3;
    end
    % Paidlunch?
    switch spec_details{3}
        case 'paidlunch'
            paidlunches = 1;
        case 'freelunch'
            paidlunches = 0;
    end
    
    % Adjust plot's yaxis for paper and slides
    switch sp
        case 'benchmark_r1_all',      overwrite_y_lim = 0.4;  % fig I.i
        case 'benchmark_r2_all',      overwrite_y_lim = 0.2;  % fig I.ii
        case 'benchmark_r3_all',      overwrite_y_lim = 0.16; % fig I.iii
        case 'competitive_r1_all',    overwrite_y_lim = 0.5;  % fig I.iv
        case 'noncompetitive_r1_all', overwrite_y_lim = 0.35; % fig I.v
        case 'placebo12_r1_all',      overwrite_y_lim = 0.45; % fig I.vi
        case 'just23_r1_all',         overwrite_y_lim = 0.35; % fig D.I.i (not in table V)
        case 'benchmark_r1_paidlunch',overwrite_y_lim = 0.45; % fig D.I.ii
        case 'benchmark_r1_freelunch',overwrite_y_lim = 0.4;  % fig D.I.iii
        case 'placebo3plus_r1_all',   overwrite_y_lim = 0.14; % fig D.I.i (not in table V)
    end

    % Run RD
    clearvars -except tasks do_* use_ranks nbins order keep_dranks dmix drop_bad_benchmark sp which_schools paidlunches overwrite_y_lim
    load ../../estimation/data;
    years = 2004:2008; % Which years to include

    rd;
    
end